from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase, SlugKey
from sqlalchemy import Index, UniqueConstraint
from sqlalchemy.dialects.postgresql import VARCHAR
from sqlalchemy.orm import Mapped, mapped_column, relationship, declared_attr

if TYPE_CHECKING:
    from .product import Product


class OriginPlatform(BigIntAuditBase, SlugKey):
    @declared_attr.directive
    @classmethod
    def __table_args__(cls):
        return (
            UniqueConstraint("name", name="uq_origin_platform_name"),
            Index("idx_origin_platform_name", "name"),
            UniqueConstraint(
                cls.slug,
                name=f"uq_{cls.__tablename__}_slug",
            ).ddl_if(callable_=cls._create_unique_slug_constraint),
            Index(
                f"ix_{cls.__tablename__}_slug_unique",
                cls.slug,
                unique=True,
            ).ddl_if(callable_=cls._create_unique_slug_index),
        )

    name: Mapped[str] = mapped_column(
        VARCHAR(255),
        nullable=False
    )

    products: Mapped[list[Product]] = relationship(
        back_populates="origin_platform",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )



from .schemas import (
    PurchaseOptionsResponse,
    CartAddRequest,
    CartProduct,
    CartAddResponse,
    CartListResponse,
    UpdatePaymentStatusRequest,
    PaymentStatusResponse,
    OptionTypes,
    PreliminaryOption,
    CheckParamsRequest,
    CheckParamsResponse,
    PurchaseOptionsRequest,
    CartListRequest,
    InitPaymentRequest,
    PaymentStatusRequest,
)

__all__ = (
    "PurchaseOptionsResponse",
    "CartAddRequest",
    "CartProduct",
    "CartAddResponse",
    "CartListResponse",
    "UpdatePaymentStatusRequest",
    "PaymentStatusResponse",
    "OptionTypes",
    "PreliminaryOption",
    "CheckParamsRequest",
    "CheckParamsResponse",
    "PurchaseOptionsRequest",
    "CartListRequest",
    "InitPaymentRequest",
    "PaymentStatusRequest",
)



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Index, UniqueConstraint
from sqlalchemy.dialects.postgresql import BIGINT
from sqlalchemy.orm import Mapped, relationship, mapped_column

if TYPE_CHECKING:
    from .category import Category
    from .product_platform import ProductPlatform


class ProductPlatformCategory(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_category_product_platform_id", "product_platform_id"),
        Index("idx_product_platform_category_category_id", "category_id"),
        UniqueConstraint("product_platform_id", "category_id", name="uq_pr_pl_category_product_platform_id_category_id"),
    )

    product_platform_id: Mapped[int] = mapped_column(
        BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"), nullable=False
    )
    category_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("category.id", ondelete="CASCADE"), nullable=False)
    version: Mapped[int] = mapped_column(BIGINT, default=1)

    product_platform: Mapped[ProductPlatform] = relationship(
        back_populates="categories",
        foreign_keys="ProductPlatformCategory.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="joined",
    )

    category: Mapped[Category] = relationship(
        back_populates="product_platform_info_categories",
        foreign_keys="ProductPlatformCategory.category_id",
        innerjoin=True,
        uselist=False,
        lazy="joined",
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import Column, ForeignKey, Index
from sqlalchemy.dialects.postgresql import BIGINT, VARCHAR, ARRAY
from sqlalchemy.orm import Mapped, relationship, mapped_column

from .decorator.jsonb_dict import JSONBDict

if TYPE_CHECKING:
    from .product_platform import ProductPlatform


class ProductPlatformVersioning(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_versioning_product_platform_id", "product_platform_id"),
    )

    product_platform_id: Mapped[int] = mapped_column(
        BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"), nullable=False, unique=True
    )
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    history: Mapped[list[ProductPlatform]] = Column(ARRAY(JSONBDict), default=[])

    product_platform: Mapped[ProductPlatform] = relationship(
        back_populates="versions",
        foreign_keys="ProductPlatformVersioning.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="noload",
    )



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct


from .product_platform_schema import ProductPlatformResponse


class ProductPlatformVersioningCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    version: Annotated[int, Parameter(title="Version")]
    history: Annotated[list[ProductPlatformResponse], Parameter(title="History")]


class ProductPlatformVersioningUpdateRequest(Struct, forbid_unknown_fields=True):
    version: Annotated[int, Parameter(title="Version")]
    history: Annotated[list[ProductPlatformResponse], Parameter(title="History")]


class ProductPlatformVersioningResponse(Struct):
    id: Annotated[int, Parameter(title="ID")]
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    version: Annotated[int, Parameter(title="Version")]
    history: Annotated[list[ProductPlatformResponse], Parameter(title="History")]

    created_at: Annotated[datetime, Parameter(title="Created At")]
    updated_at: Annotated[datetime, Parameter(title="Updated At")]



from .schemas import (
    CategoriesResponse,
    SaleInfo,
    SellerProductsRequest,
    ProductResponse,
    ProductsResponse,
    ProductDescription,
    ProductsDescriptionResponse,
    PreviewImage,
    TextProduct,
    FileProduct,
    Product,
    ProductDescriptionResponse,
    ProductPriceData,
    ProductPriceResponse,
    ProductReview,
    ProductReviewsResponse,
    ProductRow,
    SellerProductsResponse,
    ProductDiscountRequest,
    ProductDiscountInfo,
    DiscountInfo,
    ProductDiscountResponse,
    ProductSearchInfo,
    ProductSearchResponse,
    CategoriesRequest,
    ProductsRequest,
    ProductsDescriptionRequest,
    SortOrder,
    ProductDescriptionRequest,
    ProductPriceRequest,
    ProductReviewsRequest,
    ProductSearchRequest,
    ProductImageRequest,
    ProductImageResponse,
    CloneProductRequest,
    CloneProductResponse,
    SellerOfferProductsRequest,
    SellerOfferProductsResponse,
    CurrencyProductDiscount,
    Snippet,
    PageInfo,
)


__all__ = (
    "CategoriesResponse",
    "SaleInfo",
    "SellerProductsRequest",
    "ProductResponse",
    "ProductsResponse",
    "ProductDescription",
    "ProductsDescriptionResponse",
    "PreviewImage",
    "TextProduct",
    "FileProduct",
    "Product",
    "ProductDescriptionResponse",
    "ProductPriceData",
    "ProductPriceResponse",
    "ProductReview",
    "ProductReviewsResponse",
    "ProductRow",
    "SellerProductsResponse",
    "ProductDiscountRequest",
    "ProductDiscountInfo",
    "DiscountInfo",
    "ProductDiscountResponse",
    "ProductSearchInfo",
    "ProductSearchResponse",
    "CategoriesRequest",
    "ProductsRequest",
    "ProductsDescriptionRequest",
    "SortOrder",
    "ProductDescriptionRequest",
    "ProductPriceRequest",
    "ProductReviewsRequest",
    "ProductSearchRequest",
    "ProductImageRequest",
    "ProductImageResponse",
    "CloneProductRequest",
    "CloneProductResponse",
    "SellerOfferProductsRequest",
    "SellerOfferProductsResponse",
    "CurrencyProductDiscount",
    "Snippet",
    "PageInfo",
)



from datetime import datetime
from typing import Annotated, Dict, Any

from litestar.params import Parameter
from msgspec import Struct

from stofory_sdk.catalog.models.enums import SyncTaskStatus, SyncTaskType
from stofory_sdk.catalog.schemas import ProductPlatformResponse


class TaskMetaChange(Struct):
    table_name: str
    id: int
    operation: SyncTaskType


class TaskMeta(Struct):
    product_platform: ProductPlatformResponse
    changes: list[TaskMetaChange]


class ProductPlatformSyncTaskCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    task_type: Annotated[
        SyncTaskType,
        Parameter(
            title="Task Type",
            description="The task type."
        )
    ]
    task_meta: Annotated[
        TaskMeta | None,
        Parameter(
            required=False,
            title="Task Meta",
            description="The task meta."
        )
    ]
    status: Annotated[
        SyncTaskStatus,
        Parameter(
            title="Status",
            description="The status."
        )
    ]
    routing_keys: Annotated[
        list[str],
        Parameter(
            title="Routing Keys",
            description="The consumer queue`s routing keys."
        )
    ] = []
    sub_statuses: Annotated[
        Dict[str, SyncTaskStatus],
        Parameter(
            title="Sub Statuses",
            description="The sub statuses."
        )
    ] = {}
    sub_infos: Annotated[
        Dict[str, str],
        Parameter(
            title="Sub Infos",
            description="The sub infos."
        )
    ] = {}
    info: Annotated[
        str | None,
        Parameter(
            required=False,
            title="Info",
            description="The info."
        )
    ] = None


class ProductPlatformSyncTaskUpdateRequest(Struct, forbid_unknown_fields=True):
    task_type: Annotated[
        SyncTaskType,
        Parameter(
            title="Task Type",
            description="The task type."
        )
    ]
    task_meta: Annotated[
        TaskMeta | None,
        Parameter(
            title="Task Meta",
            description="The task meta."
        )
    ]
    routing_keys: Annotated[
        list[str],
        Parameter(
            title="Routing Keys",
            description="The consumer queue`s routing keys."
        )
    ]
    sub_statuses: Annotated[
        Dict[str, SyncTaskStatus],
        Parameter(
            title="Sub Statuses",
            description="The sub statuses."
        )
    ]
    sub_infos: Annotated[
        Dict[str, str],
        Parameter(
            title="Sub Infos",
            description="The sub infos."
        )
    ]
    status: Annotated[
        SyncTaskStatus,
        Parameter(
            title="Status",
            description="The status."
        )
    ]
    info: Annotated[
        str | None,
        Parameter(
            title="Info",
            description="The info."
        )
    ]


class ProductPlatformSyncTaskResponse(Struct):
    id: Annotated[int, Parameter(title="ID")]
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    task_type: Annotated[
        SyncTaskType,
        Parameter(
            title="Task Type",
            description="The task type."
        )
    ]
    task_meta: Annotated[
        TaskMeta | None,
        Parameter(
            title="Task Meta",
            description="The task meta."
        )
    ]
    routing_keys: Annotated[
        list[str],
        Parameter(
            title="Routing Keys",
            description="The consumer queue`s routing keys."
        )
    ]
    sub_statuses: Annotated[
        Dict[str, SyncTaskStatus],
        Parameter(
            title="Sub Statuses",
            description="The sub statuses."
        )
    ]
    sub_infos: Annotated[
        Dict[str, str],
        Parameter(
            title="Sub Infos",
            description="The sub infos."
        )
    ]
    status: Annotated[
        SyncTaskStatus,
        Parameter(
            title="Status",
            description="The status."
        )
    ]
    info: Annotated[
        str | None,
        Parameter(
            required=False,
            title="Info",
            description="The info."
        )
    ]

    created_at: Annotated[datetime, Parameter(title="Created At")]
    updated_at: Annotated[datetime, Parameter(title="Updated At")]



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct, Meta

from stofory_sdk.catalog.models.enums import SyncTaskType, SyncTaskPriority


class SyncQueueMappingCreateRequest(Struct, forbid_unknown_fields=True):
    priority: Annotated[SyncTaskPriority, Parameter(title="Priority", description="The priority of the sync task.")]
    consumer: Annotated[str, Parameter(title="Consumer Name", description="The prefix of the consumer."), Meta(min_length=1)]
    table_names: Annotated[list[str], Parameter(title="Table Names", description="The names of the tables."), Meta(min_length=1)]
    operation: Annotated[SyncTaskType, Parameter(title="Operation", description="The operation of the sync task.")]


class SyncQueueMappingUpdateRequest(Struct, forbid_unknown_fields=True):
    priority: Annotated[SyncTaskPriority, Parameter(title="Priority", description="The priority of the sync task.")]
    consumer: Annotated[str, Parameter(title="Consumer Name", description="The prefix of the consumer."), Meta(min_length=1)]
    table_names: Annotated[list[str], Parameter(title="Table Names", description="The names of the tables."), Meta(min_length=1)]
    operation: Annotated[SyncTaskType, Parameter(title="Operation", description="The operation of the sync task.")]


class SyncQueueMappingResponse(Struct):
    id: int
    priority: SyncTaskPriority
    consumer: str
    table_names: list[str]
    operation: SyncTaskType

    created_at: datetime
    updated_at: datetime



import enum


class ProductType(enum.StrEnum):
    DLC = enum.auto()  # "Дополнения (DLC)"
    GAME = enum.auto()  # "Игры (Games)"
    GAME_ITEMS = enum.auto()  # "Игровые вещи (Game Items)"
    GIFT_CARDS = enum.auto()  # "Подарочные карты (Gift Cards)"
    OTHERS = enum.auto()  # "Другое (Others)"
    PRE_ORDERS = enum.auto()  # "Предзаказы (Pre-Orders)"
    SOFTWARE = enum.auto()  # "Программное обеспечение (Software)"
    SUBSCRIPTIONS = enum.auto()  # "Подписки (Subscriptions/Subs)"
    TOP_UP = enum.auto()  # "Прямое пополнение (Top-Up)"



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct

from stofory_sdk.catalog.models.enums import Locale


class LocalizationCreateRequest(Struct, forbid_unknown_fields=True):
    locale: Annotated[
        Locale,
        Parameter(
            title="Locale",
            description="The locale of the localization."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="The enabled status of the localization."
        )
    ]


class LocalizationUpdateRequest(Struct, forbid_unknown_fields=True):
    locale: Annotated[
        Locale,
        Parameter(
            title="Locale",
            description="The locale of the localization."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="The enabled status of the localization."
        )
    ]


class LocalizationResponse(Struct):
    id: int
    locale: Locale
    is_enabled: bool
    created_at: datetime
    updated_at: datetime



import enum
from typing import Dict, List, TypeVar, Generic, get_origin, get_args, Type

from stofory_sdk.catalog.models.enums import Locale as AdminLocale, Currency as AdminCurrency, \
    ParameterType as AdminParameterType, ModifierOperator as AdminModifierOperator

from stofory_sdk.digiseller.schemas import Locale as DigisellerLocale, Currency as DigisellerCurrency, \
    OptionTypes as DigisellerOptionTypes, VariantTypes as DigisellerVariantTypes

__all__ = [
    "LocaleMapper",
    "ReversedLocaleMapper",
    "CurrencyMapper",
    "ParameterTypeMapper",
    "ModifierOperatorMapper",
    "ReversedCurrencyMapper",
    "ReversedParameterTypeMapper",
]

T = TypeVar('T')
U = TypeVar('U')


class EnumMapper(Generic[T, U]):
    model: str
    mapping: Dict[T, U]
    key_type: Type[T]
    value_type: Type[U]

    def __init__(self, mapping: Dict[T, U]):
        if not isinstance(mapping, dict):
            raise ValueError("Mapping must be a dictionary")

        self.key_type, self.value_type = get_args(self.__orig_bases__[0])
        # print(self.key_type, self.value_type)
        # print(type(self.key_type), type(self.value_type))

        if not all(isinstance(k, self.key_type) for k in mapping.keys()):
            raise ValueError(f"Keys must be of type {self.key_type.__class__}")

        if not all(isinstance(v, self.value_type) for v in mapping.values()):
            raise ValueError(f"Values must be of type {self.value_type}")

        self.mapping = mapping

    def add(self, key: T, value: U) -> bool:
        if not isinstance(key, self.key_type):
            raise ValueError(f"Key must be of type {self.key_type}")

        if not isinstance(value, self.value_type):
            raise ValueError(f"Value must be of type {self.value_type}")

        self.mapping[key] = value
        return True

    def get(self, key: T) -> U:
        if not isinstance(key, self.key_type):
            raise ValueError(f"Key must be of type {self.key_type}")

        if key not in self.mapping:
            raise KeyError(f"Key {key} not found in {self.model}`s mapping")

        return self.mapping[key]


class LocaleMapper(EnumMapper[AdminLocale, DigisellerLocale]):
    model = "Locale"

    def __init__(self):
        super().__init__({
            AdminLocale.RU_RU: DigisellerLocale.RU_RU,
            AdminLocale.EN_EN: DigisellerLocale.EN_US,
        })


class ReversedLocaleMapper(EnumMapper[DigisellerLocale, AdminLocale]):
    model = "Locale"

    def __init__(self):
        super().__init__({
            DigisellerLocale.RU_RU: AdminLocale.RU_RU,
            DigisellerLocale.EN_US: AdminLocale.EN_EN,
            DigisellerLocale.EN_EN: AdminLocale.EN_EN,
        })


class CurrencyMapper(EnumMapper[AdminCurrency, DigisellerCurrency]):
    model = "Currency"

    def __init__(self):
        super().__init__({
            AdminCurrency.RUB: DigisellerCurrency.RUB,
            # AdminCurrency.KZT: DigisellerCurrency.KZT,
            AdminCurrency.USD: DigisellerCurrency.USD,
        })


class ReversedCurrencyMapper(EnumMapper[DigisellerCurrency, AdminCurrency]):
    model = "Currency"

    def __init__(self):
        super().__init__({
            DigisellerCurrency.RUB: AdminCurrency.RUB,
            DigisellerCurrency.USD: AdminCurrency.USD,
        })


class ParameterTypeMapper(EnumMapper[AdminParameterType, DigisellerOptionTypes]):
    model = "ParameterType"

    def __init__(self):
        super().__init__({
            AdminParameterType.TEXT: DigisellerOptionTypes.TEXT,
            AdminParameterType.RADIO: DigisellerOptionTypes.RADIO,
            AdminParameterType.CHECKBOX: DigisellerOptionTypes.CHECKBOX,
        })


class ReversedParameterTypeMapper(EnumMapper[DigisellerOptionTypes, AdminParameterType]):
    model = "ParameterType"

    def __init__(self):
        super().__init__({
            DigisellerOptionTypes.TEXT: AdminParameterType.TEXT,
            DigisellerOptionTypes.RADIO: AdminParameterType.RADIO,
            DigisellerOptionTypes.CHECKBOX: AdminParameterType.CHECKBOX,
        })


class ModifierOperatorMapper(EnumMapper[AdminModifierOperator, DigisellerVariantTypes]):
    model = "ModifierOperator"

    def __init__(self):
        super().__init__({
            AdminModifierOperator.PLUS: DigisellerVariantTypes.PRICEPLUS,
            AdminModifierOperator.MINUS: DigisellerVariantTypes.PRICEMINUS,
            AdminModifierOperator.PLUS_PERCENT: DigisellerVariantTypes.PERCENTPLUS,
            AdminModifierOperator.MINUS_PERCENT: DigisellerVariantTypes.PERCENTMINUS,
        })



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct


class ProductPlatformImageCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    relative_path: Annotated[
        str,
        Parameter(
            title="Relative Path",
            description="The relative path of the image."
        )
    ]
    domain: Annotated[
        str,
        Parameter(
            title="Domain",
            description="The domain of the image."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="Whether the image is enabled."
        )
    ]
    order: Annotated[
        int,
        Parameter(
            title="Order",
            description="The order of the image."
        )
    ]


class ProductPlatformImageUpdateRequest(Struct, forbid_unknown_fields=True):
    relative_path: Annotated[
        str,
        Parameter(
            title="Relative Path",
            description="The relative path of the image."
        )
    ]
    domain: Annotated[
        str,
        Parameter(
            title="Domain",
            description="The domain of the image."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="Whether the image is enabled."
        )
    ]
    order: Annotated[
        int,
        Parameter(
            title="Order",
            description="The order of the image."
        )
    ]


class ProductPlatformImageResponse(Struct):
    id: int
    product_platform_id: int
    version: int
    relative_path: str
    domain: str
    is_enabled: bool
    order: int
    created_at: datetime
    updated_at: datetime


class ProductPlatformImageUploadResponse(Struct):
    relative_path: str
    domain: str



from __future__ import annotations

from typing import TYPE_CHECKING, Dict, Any


from advanced_alchemy.base import BigIntAuditBase, SlugKey
from sqlalchemy.dialects.postgresql import VARCHAR, TEXT, JSONB, BOOLEAN
from sqlalchemy.orm import Mapped, relationship, mapped_column

from .decorator.jsonb_dict import JSONBDict

if TYPE_CHECKING:
    from .product_platform import ProductPlatform
    from .product_platform_external_connection import ProductPlatformExternalConnection


class Platform(BigIntAuditBase, SlugKey):
    name: Mapped[str] = mapped_column(VARCHAR(255), nullable=False)
    description: Mapped[str] = mapped_column(TEXT(), nullable=False)
    custom_fields_template: Mapped[Dict[str, Any]] = mapped_column(JSONBDict, default={})
    is_external: Mapped[bool] = mapped_column(BOOLEAN, default=False)

    product_platforms: Mapped[list[ProductPlatform]] = relationship(
        back_populates="platform",
        lazy="selectin",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    external_connections: Mapped[list[ProductPlatformExternalConnection]] = relationship(
        back_populates="platform",
        lazy="selectin",
        uselist=True,
        # cascade="all, delete"
    )



from enum import StrEnum
import msgspec


# Перечисление для локализации
class Locale(StrEnum):
    """Язык отображения информации."""
    RU_RU = "ru-RU"
    EN_EN = "en-EN"
    EN_US = "en-US"


# Перечисление для типов валют
class Currency(StrEnum):
    """Тип валюты для отображения стоимости товара."""
    RUB = "RUB"  # Российский рубль
    RUR = "RUR"  # Российский рубль

    USD = "USD"  # Доллар США
    EUR = "EUR"  # Евро
    UAH = "UAH"  # Украинская гривна


# Перечисление для типов валют дигиселлера
class CurrencyDigiseller(StrEnum):
    WMZ = 'WMZ'
    WMR = 'WMR'
    WME = 'WME'
    WMX = 'WMX'


# Перечисление для типов опций параметра
class VariantTypes(StrEnum):
    PERCENTPLUS = 'percentplus'
    PERCENTMINUS = 'percentminus'
    PRICEPLUS = 'priceplus'
    PRICEMINUS = 'priceminus'


# Перечисление для типов параметров
class OptionTypes(StrEnum):
    TEXTAREA = "textarea"
    CHECKBOX = "checkbox"
    TEXT = "text"
    RADIO = "radio"
    SELECT = "select"


# Схема сообщения об ошибке
class ErrorMessage(msgspec.Struct, forbid_unknown_fields=True):
    locale: Locale
    value: str


# Схема ошибки
class Error(msgspec.Struct, forbid_unknown_fields=True):
    code: str
    message: list[ErrorMessage] | None


# Базовая схема ответа
class BaseResponse(msgspec.Struct, forbid_unknown_fields=True):
    retval: int
    retdesc: str | None
    errors: list[Error] | None



import enum
from typing import List

import msgspec

from stofory_sdk.catalog.models.enums import ParameterType, ModifierOperator, Currency
from stofory_sdk.digiseller.n2_payments.schemas import CartTypeCurrency


class OptionSchema(msgspec.Struct, forbid_unknown_fields=True):
    option_id: int


class ParameterSchema(msgspec.Struct, forbid_unknown_fields=True):
    parameter_id: int
    parameter_type: ParameterType
    value: str
    options: List[OptionSchema]


class CartProductSchema(msgspec.Struct, forbid_unknown_fields=True):
    id: int
    name: str
    discount: int
    price: int
    total_price: int | None
    quantity: int
    unit_quantity: int | None
    payment_type: CartTypeCurrency
    parameters: List[ParameterSchema]


class CartCreateRequest(msgspec.Struct, forbid_unknown_fields=True):
    email: str | None
    items: List[CartProductSchema]


class CartCreateResponse(msgspec.Struct, forbid_unknown_fields=True):
    cart_id: str
    id_po: int | None



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Index, UniqueConstraint
from sqlalchemy.dialects.postgresql import BIGINT, BOOLEAN
from sqlalchemy.orm import Mapped, relationship, mapped_column

from .decorator.jsonb_dict import JSONBDict

if TYPE_CHECKING:
    from .product import Product
    from .platform import Platform

    from .product_platform_pricing import ProductPlatformPricing
    from .product_platform_bonus import ProductPlatformBonus
    from .product_platform_category import ProductPlatformCategory
    from .product_platform_description import ProductPlatformDescription
    from .product_platform_name import ProductPlatformName
    from .product_platform_discount import ProductPlatformDiscount
    from .product_platform_guarantee import ProductPlatformGuarantee

    from .product_platform_parameter import ProductPlatformParameter
    from .product_platform_image import ProductPlatformImage
    from .product_platform_video import ProductPlatformVideo

    from .product_platform_sync_task import ProductPlatformSyncTask
    from .product_platform_versioning import ProductPlatformVersioning
    from .product_platform_external_connection import ProductPlatformExternalConnection

    from .product_platform_content import ProductPlatformContent



class ProductPlatform(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_product_id", "product_id"),
        Index("idx_product_platform_platform_id", "platform_id"),
        UniqueConstraint("product_id", "platform_id", name="uq_product_platform_product_id_platform_id"),
    )

    product_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product.id", ondelete="CASCADE"), nullable=False)
    platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("platform.id", ondelete="CASCADE"), nullable=False)
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    is_enabled: Mapped[bool] = mapped_column(BOOLEAN, default=False)
    is_visible: Mapped[bool] = mapped_column(BOOLEAN, default=False)
    custom_fields: Mapped[dict] = mapped_column(JSONBDict, default={})

    product: Mapped[Product] = relationship(
        back_populates="product_platforms",
        foreign_keys="ProductPlatform.product_id",
        innerjoin=True,
        uselist=False,
        lazy="noload",   # select/noload?
    )

    platform: Mapped[Platform] = relationship(
        back_populates="product_platforms",
        foreign_keys="ProductPlatform.platform_id",
        innerjoin=True,
        uselist=False,
        lazy="noload",
    )

    names: Mapped[list[ProductPlatformName]] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ProductPlatformName.id)"
    )

    descriptions: Mapped[list[ProductPlatformDescription]] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ProductPlatformDescription.id)"
    )

    categories: Mapped[list[ProductPlatformCategory]] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ProductPlatformCategory.id)"
    )

    parameters: Mapped[list[ProductPlatformParameter]] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ProductPlatformParameter.id)"
    )

    images: Mapped[list[ProductPlatformImage]] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ProductPlatformImage.id)"
    )

    videos: Mapped[list[ProductPlatformVideo]] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ProductPlatformVideo.id)"
    )

    price: Mapped[ProductPlatformPricing] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=False,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    bonus: Mapped[ProductPlatformBonus] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=False,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    discount: Mapped[ProductPlatformDiscount] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=False,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    guarantee: Mapped[ProductPlatformGuarantee] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=False,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    sync_tasks: Mapped[list[ProductPlatformSyncTask]] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ProductPlatformSyncTask.id)"
    )

    versions: Mapped[ProductPlatformVersioning] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=False,
        passive_deletes=True,
        cascade="all, delete-orphan",
    )

    external_connections: Mapped[ProductPlatformExternalConnection] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=False,
        passive_deletes=True,
        cascade="all, delete-orphan",
    )

    contents: Mapped[list[ProductPlatformContent]] = relationship(
        back_populates="product_platform",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ProductPlatformContent.id)"
    )



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct

from stofory_sdk.catalog.models.enums import ParameterType

from .parameter_name_schema import ParameterNameResponse
from .parameter_comment_schema import ParameterCommentResponse
from .parameter_option_schema import ParameterOptionResponse


class ProductPlatformParameterCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    parameter_type: Annotated[
        ParameterType,
        Parameter(
            title="Parameter Type",
            description="The type of the parameter to create."
        )
    ]
    is_required: Annotated[
        bool,
        Parameter(
            title="Is Required",
            description="Whether the parameter is required."
        )
    ]
    order: Annotated[
        int,
        Parameter(
            title="Order",
            description="The order of the parameter."
        )
    ]


class ProductPlatformParameterUpdateRequest(Struct, forbid_unknown_fields=True):
    parameter_type: Annotated[
        ParameterType,
        Parameter(
            title="Parameter Type",
            description="The type of the parameter to update.",
        )
    ]
    is_required: Annotated[
        bool,
        Parameter(
            title="Is Required",
            description="Whether the parameter is required.",
        )
    ]
    order: Annotated[
        int,
        Parameter(
            title="Order",
            description="The order of the parameter.",
        )
    ]


class ProductPlatformParameterResponse(Struct):
    id: int
    product_platform_id: int
    version: int
    parameter_type: ParameterType
    is_required: bool
    order: int
    created_at: datetime
    updated_at: datetime

    names: list[ParameterNameResponse]
    comments: list[ParameterCommentResponse]
    options: list[ParameterOptionResponse]


class ProductPlatformParameterShortenedResponse(Struct):
    id: int
    name: str | None



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import Index, ForeignKey
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy.dialects.postgresql import BIGINT, TEXT, ENUM

from .enums import ContentType

if TYPE_CHECKING:
    from .product_platform import ProductPlatform


class ProductPlatformContent(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_content_product_platform_id", "product_platform_id"),
    )

    product_platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"),
                                                     nullable=False)
    content_type: Mapped[ContentType] = mapped_column(ENUM(ContentType, name="content_type_enum"), nullable=False)
    value: Mapped[str] = mapped_column(TEXT, nullable=False)    # Could be actual content or URL to file

    product_platform: Mapped[ProductPlatform] = relationship(
        back_populates="contents",
        foreign_keys="ProductPlatformContent.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="noload"
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Index
from sqlalchemy.dialects.postgresql import BOOLEAN, BIGINT, VARCHAR, SMALLINT
from sqlalchemy.orm import Mapped, relationship, mapped_column


if TYPE_CHECKING:
    from .product_platform import ProductPlatform


class ProductPlatformImage(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_image_product_platform_id", "product_platform_id"),
    )

    product_platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"), nullable=False)
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    relative_path: Mapped[str] = mapped_column(VARCHAR(255), nullable=False)
    domain: Mapped[str] = mapped_column(VARCHAR(255), nullable=False)
    is_enabled: Mapped[bool] = mapped_column(BOOLEAN, default=True)
    order: Mapped[int] = mapped_column(SMALLINT, nullable=False)

    product_platform: Mapped[ProductPlatform] = relationship(
        back_populates="images",
        foreign_keys="ProductPlatformImage.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="joined",
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Index
from sqlalchemy.dialects.postgresql import BIGINT, VARCHAR
from sqlalchemy.orm import Mapped, relationship, mapped_column

if TYPE_CHECKING:
    from .product_platform import ProductPlatform


class ProductPlatformVideo(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_video_product_platform_id", "product_platform_id"),
    )

    product_platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"), nullable=False)
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    url: Mapped[str] = mapped_column(VARCHAR(255), nullable=False)
    platform: Mapped[str] = mapped_column(VARCHAR(50), nullable=False)

    product_platform: Mapped[ProductPlatform] = relationship(
        back_populates="videos",
        foreign_keys="ProductPlatformVideo.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="joined"
    )



from typing import Dict, Literal, List, Union, Optional, Any, TypeVar
import msgspec


__all__ = ('SortType', 'FieldFilter', 'SortingRulesSchema', 'FilteringRulesSchema')
T = TypeVar("T", bool, str, int, float)

SortType = Literal["asc", "desc"]


class FieldFilter[T](msgspec.Struct, forbid_unknown_fields=True):
    and_: Optional[Literal[1, 0]] = None
    ge: Optional[T] = None
    gt: Optional[T] = None
    le: Optional[T] = None
    lt: Optional[T] = None
    e: Optional[T] = None
    like: Optional[T] = None
    ilike: Optional[T] = None

    def __post_init__(self):
        self.validate_filters()

    def validate_filters(self):
        """Validate the filter rules."""
        non_empty_fields = {key for key, value in msgspec.structs.asdict(self).items() if
                            value is not None and key != "and_"}
        if self.and_ is None:
            if len(non_empty_fields) > 1:
                raise ValueError("Field 'and_' must be specified if more than one filter is present.")
        elif len(non_empty_fields) == 1:
            raise ValueError("Field 'and_' must be empty if exactly one filter is present.")
        return self


class SortingRulesSchema(msgspec.Struct, kw_only=True):
    sorting_order: Optional[Union[str, List[str]]] = None
    __sorting_rules: Dict[str, SortType] = msgspec.field(default_factory=dict)

    def __post_init__(self):
        """Populate sorting rules based on the sorting order and fields."""
        sorting_order = self.sorting_order
        if isinstance(sorting_order, str):
            sorting_order = sorting_order.split(",")

        data = {}

        for key, value in msgspec.to_builtins(self).items():
            if isinstance(value, str) and value.strip().lower() in {"asc", "desc"}:
                new_key = key.split("s__")
                if len(new_key) == 2:
                    data[new_key[1]] = value.strip().lower()

        if sorting_order:
            sorted_data = {}
            for key in sorting_order:
                if key in data:
                    sorted_data[key] = data[key]
            for key in data:
                if key not in sorted_data:
                    sorted_data[key] = data[key]
            self.__sorting_rules = sorted_data
        else:
            self.__sorting_rules = data

    def get_sorting_rules(self) -> Dict[str, SortType]:
        """Retrieve the populated sorting rules."""
        return self.__sorting_rules


class FilteringRulesSchema(msgspec.Struct):
    __filter_rules: Dict[str, Dict[str, Any]] = msgspec.field(default_factory=dict)

    def __post_init__(self):
        """Populate filter rules."""
        filters = {}
        # msgspec.structs.asdict(self).items()
        # for key, value in msgspec.to_builtins(self).items():
        #     if isinstance(value, dict):
        #         for local_key, local_value in value.items():
        #             if local_key in set(msgspec.structs.asdict(FieldFilter()).keys()):
        #                 new_key = key.split("f__")
        #                 if len(new_key) == 2:
        #                     filters[new_key[1]] = value
        for key, value in msgspec.structs.asdict(self).items():
            if isinstance(value, FieldFilter):
                value.validate_filters()
                new_key = key.split("f__")
                if len(new_key) == 2:
                    filters[new_key[1]] = value
        self.__filter_rules = filters

    def get_filter_rules(self) -> Dict[str, Dict[str, Any]]:
        """Retrieve the populated filter rules."""
        return self.__filter_rules


class AreasOfResponsibilitiesQueryParamsSchemas(FilteringRulesSchema, kw_only=True):
    f__visibility: Optional[FieldFilter[bool]] = None
    f__coefficient: Optional[FieldFilter[float]] = None


# Usage example
schema = AreasOfResponsibilitiesQueryParamsSchemas(
    f__visibility=FieldFilter(e=True)
)
# print(schema)
schema.get_filter_rules()
# print("Sorting Rules:", schema.get_filter_rules())



from abc import ABC, abstractmethod
from typing import Any

import msgspec


class BaseCustomSchemaType(ABC):

    @abstractmethod
    def _validate(self):
        pass

    @abstractmethod
    def into_encoding_type(self) -> Any:
        pass

    @abstractmethod
    def into_decoding_type(self) -> Any:
        pass

    @abstractmethod
    def value(self) -> Any:
        pass

    @classmethod
    @abstractmethod
    def from_inner_type(cls, value: Any) -> Any:
        pass

    def __repr__(self):
        return str(self.value())


class BaseStruct(msgspec.Struct):
    def to_dict(self, exclude: list[str] = None) -> dict[str, Any]:
        exclude = exclude or []

        result_dict = {f: getattr(self, f) for f in self.__struct_fields__ if
                f not in exclude and getattr(self, f, None) != msgspec.UNSET}

        for k, v in result_dict.items():
            if isinstance(v, BaseCustomSchemaType):
                result_dict[k] = v.into_encoding_type()

        return result_dict


from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct


class ProductPlatformExternalConnectionCreateRequest(Struct, forbid_unknown_fields=True):
    product_id: Annotated[int, Parameter(title="Product ID")]
    platform_id: Annotated[int, Parameter(title="Platform ID")]
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    version: Annotated[int, Parameter(title="Version")]
    external_id: Annotated[str, Parameter(title="External ID")]
    external_meta: Annotated[dict, Parameter(title="External Meta")]


class ProductPlatformExternalConnectionUpdateRequest(Struct, forbid_unknown_fields=True):
    version: Annotated[int, Parameter(title="Version")]
    external_id: Annotated[str, Parameter(title="External ID")]
    external_meta: Annotated[dict, Parameter(title="External Meta")]


class ProductPlatformExternalConnectionResponse(Struct):
    id: Annotated[int, Parameter(title="ID")]
    product_id: Annotated[int, Parameter(title="Product ID")]
    platform_id: Annotated[int, Parameter(title="Platform ID")]
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    version: Annotated[int, Parameter(title="Version")]
    external_id: Annotated[str, Parameter(title="External ID")]
    external_meta: Annotated[dict, Parameter(title="External Meta")]

    created_at: Annotated[datetime, Parameter(title="Created At")]
    updated_at: Annotated[datetime, Parameter(title="Updated At")]



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct

from .category_schema import CategoryResponse


class ProductPlatformCategoryCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[
        int,
        Parameter(
            title="Product Platform ID",
            description="The ID of the product platform."
        )
    ]
    category_id: Annotated[
        int,
        Parameter(
            title="Category ID",
            description="The ID of the category."
        )
    ]


class ProductPlatformCategoryUpdateRequest(Struct, forbid_unknown_fields=True):
    category_id: Annotated[
        int,
        Parameter(
            title="Category ID",
            description="The ID of the category."
        )
    ]


class ProductPlatformCategoryResponse(Struct):
    id: int
    product_platform_id: int
    version: int
    category_id: int
    created_at: datetime
    updated_at: datetime

    category: CategoryResponse



from .mappings import LocaleMapper, CurrencyMapper, ParameterTypeMapper, ModifierOperatorMapper, ReversedLocaleMapper, \
    ReversedCurrencyMapper, ReversedParameterTypeMapper

__all__ = [
    "LocaleMapper",
    "CurrencyMapper",
    "ParameterTypeMapper",
    "ModifierOperatorMapper",
    "ReversedLocaleMapper",
    "ReversedCurrencyMapper",
    "ReversedParameterTypeMapper",
]



import enum
from typing import List

import msgspec

from stofory_sdk.digiseller.schemas import BaseResponse


class ContentFileNode(msgspec.Struct, forbid_unknown_fields=True):
    content_id: int
    filename: str


class AddTextNode(msgspec.Struct, forbid_unknown_fields=True):
    value: str
    id_v: int
    serial: str | None = None


class ContentTextNode(msgspec.Struct, forbid_unknown_fields=True):
    content_id: int
    serial: str | None = None


class ContentCode(msgspec.Struct, forbid_unknown_fields=True):
    status: str


class ContentFileEditNode(msgspec.Struct, forbid_unknown_fields=True):
    # content_id: int   # в доках есть, по факту нет
    new_content_id: int
    filename: str


class ContentDeleteAll(msgspec.Struct, forbid_unknown_fields=True):
    deleted_count: int


class DeliveryTypes(enum.StrEnum):
    Email = "Email"
    JSON = "JSON"
    XML = "XML"


class ContentForm(msgspec.Struct, forbid_unknown_fields=True):
    status: str


"""
    Request/Response schemas for N7
"""


class AddTextRequest(msgspec.Struct, forbid_unknown_fields=True):
    product_id: int
    content: list[AddTextNode]


class EditTextRequest(msgspec.Struct, forbid_unknown_fields=True):
    content_id: int
    value: str
    serial: str | None = None
    update_old: bool = False


class CreateEditFormRequest(msgspec.Struct, forbid_unknown_fields=True):
    product_id: int
    address: str
    method: DeliveryTypes
    encoding: str
    options: bool
    answer: bool
    allow_purchase_multiple_items: bool
    url: str | None = None


class AddFileResponse(BaseResponse, forbid_unknown_fields=True):
    content: List[ContentFileNode] | None


class AddTextResponse(BaseResponse, forbid_unknown_fields=True):
    content: List[ContentTextNode] | None


class UpdateCodeResponse(BaseResponse, forbid_unknown_fields=True):
    content: ContentCode | None


class UpdateFileResponse(BaseResponse, forbid_unknown_fields=True):
    content: ContentFileEditNode | None


class UpdateTextResponse(BaseResponse, forbid_unknown_fields=True):
    content: ContentTextNode | None


class DeleteContentResponse(BaseResponse, forbid_unknown_fields=True):
    content: ContentDeleteAll | None


class CreateEditFormResponse(BaseResponse, forbid_unknown_fields=True):
    content: ContentForm | None


from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct

from stofory_sdk.catalog.models.enums import ModifierOperator

from .parameter_option_name_schema import ParameterOptionNameResponse


class ParameterOptionCreateRequest(Struct, forbid_unknown_fields=True, tag=True):
    parameter_id: Annotated[int, Parameter(title="Parameter ID")]
    modifier_operator: Annotated[
        ModifierOperator,
        Parameter(
            title="Modifier Operator",
            description="The modifier operator of the product parameter option."
        )
    ]
    modifier_value: Annotated[
        float,
        Parameter(
            title="Modifier Value",
            description="The modifier value of the product parameter option."
        )
    ]
    is_default: Annotated[
        bool,
        Parameter(
            title="Is Default",
            description="The default status of the product parameter option."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="The enabled status of the product parameter option."
        )
    ]
    order: Annotated[
        int,
        Parameter(
            title="Order",
            description="The order of the product parameter option."
        )
    ]


class ParameterOptionUpdateRequest(Struct, forbid_unknown_fields=True):
    modifier_operator: Annotated[
        ModifierOperator,
        Parameter(
            title="Modifier Operator",
            description="The modifier operator of the product parameter option."
        )
    ]
    modifier_value: Annotated[
        float,
        Parameter(
            title="Modifier Value",
            description="The modifier value of the product parameter option."
        )
    ]
    is_default: Annotated[
        bool,
        Parameter(
            title="Is Default",
            description="The default status of the product parameter option."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="The enabled status of the product parameter option."
        )
    ]
    order: Annotated[
        int,
        Parameter(
            title="Order",
            description="The order of the product parameter option."
        )
    ]


class ParameterOptionResponse(Struct):
    id: int
    parameter_id: int
    version: int
    modifier_operator: ModifierOperator
    modifier_value: float
    is_default: bool
    is_enabled: bool
    order: int
    created_at: datetime
    updated_at: datetime

    names: list[ParameterOptionNameResponse]



from .currency import Currency
from .locale import Locale
from .location import LocationType
from .modifier_operator import ModifierOperator
from .parameter_type import ParameterType
from .part_of_world import PartOfWorldType
from .product_kind import ProductKind
from .pricing_type import PricingType
from .product_type import ProductType
from .sync_task_type import SyncTaskType
from .sync_task_status import SyncTaskStatus
from .sync_task_priority import SyncTaskPriority
from .sync_process_status import SyncProcessStatus
from .content_type import ContentType


__all__ = (
    "Currency",
    "Locale",
    "LocationType",
    "ModifierOperator",
    "ParameterType",
    "PartOfWorldType",
    "ProductKind",
    "PricingType",
    "ProductType",
    "SyncTaskType",
    "SyncTaskStatus",
    "SyncTaskPriority",
    "SyncProcessStatus",
    "ContentType",
)



from enum import StrEnum
from typing import Any, List, Union, Optional, Annotated

import msgspec

from ..schemas import Locale, OptionTypes


# Предварительная проверка параметров
class ProductOption(msgspec.Struct, forbid_unknown_fields=True):
    id: int  # Идентификатор опции
    type: str  # Тип опции, например, текст, чекбокс и т.д.
    value: Union[int, str]  # Значение опции, может быть текстовым или числовым


class CheckParamsRequest(msgspec.Struct, forbid_unknown_fields=True):
    product: dict  # Информация о продукте, включая идентификатор и количество
    options: List[ProductOption]  # Список выбранных параметров продукта


class CheckParamsResponse(msgspec.Struct, forbid_unknown_fields=True):
    error: str  # Сообщение об ошибке, если есть; пустое, если ошибок нет


# Определение значений параметров товара при покупке

class PreliminaryOption(msgspec.Struct, forbid_unknown_fields=True):
    """Схема параметра продукта, где type представляет тип опции, а value — её значение"""
    id: int  # Идентификатор параметра
    type: OptionTypes  # Тип опции (например, текст, чекбокс)
    value: str  # Значение опции, может быть текстом или id выбранного значения для select, checkbox, radio


class IdParametersValuesPurchaseOptions(msgspec.Struct, forbid_unknown_fields=True, tag="id"):
    type = "id"  # Обязательно передавать
    id: int  # ID выбранного значения, применимо для select, checkbox, radio


class TextParametersValuesPurchaseOptions(msgspec.Struct, forbid_unknown_fields=True, tag="text"):
    type = "text"  # Обязательно передавать
    text: Optional[str]  # Текстовое значение, если параметр типа 'text'


# Основная структура с объединением
class SetParametersPurchaseOptions(msgspec.Struct, forbid_unknown_fields=True, tag_field="type"):
    id: int  # ID параметра
    value: Union[
        IdParametersValuesPurchaseOptions,
        TextParametersValuesPurchaseOptions
    ]  # Значение параметра, включая id и/или text


class PurchaseOptionsRequest(msgspec.Struct, forbid_unknown_fields=True):
    product_id: int  # Идентификатор продукта
    options: list[SetParametersPurchaseOptions] = msgspec.UNSET  # Список опций с их значениями
    unit_cnt: Optional[int] = msgspec.UNSET  # Количество товара с динамической ценой
    lang: Locale = Locale.RU_RU  # Язык отображения информации
    ip: str = "0.0.0.0"  # IP-адрес пользователя для дополнительной информации


class PurchaseOptionsResponse(msgspec.Struct, forbid_unknown_fields=True):
    retval: int  # Код выполнения запроса: 0 - успешно
    retdesc: Optional[str]  # Описание результата выполнения
    id_po: int  # Уникальный ID набора значений, используемый для перехода к оплате


# Работа с корзиной
# 1. Добавление товара в корзину

# Перечисление для типа валюты (способа платежа)
class CartTypeCurrency(StrEnum):
    WMR = "WMR"  # WebMoney рубли
    WMZ = "WMZ"  # WebMoney доллары
    WME = "WME"  # WebMoney евро
    WMX = "WMX"  # WebMoney биткоины
    PCR = "PCR"  # Промсвязьбанк
    QSP = "QSP"  # Qiwi
    PYU = "PYU"  # PayU
    RCC = "RCC"  # Российская кредитная карта
    MTS = "MTS"  # МТС
    TL2 = "TL2"  # Тел2
    BLN = "BLN"  # Билайн
    MGF = "MGF"  # Мегафон
    BNK = "BNK"  # Банковский перевод
    GRN = "GRN"  # Гривна


# Перечисление для языка интерфейса
class CartLanguage(StrEnum):
    RU_RU = "ru-RU"  # Русский
    EN_US = "en-US"  # Английский


class CartAddRequest(msgspec.Struct, forbid_unknown_fields=True):
    product_id: int  # Идентификатор продукта
    product_cnt: Annotated[int, msgspec.Meta(ge=1)]  # Количество товаров для добавления
    typecurr: CartTypeCurrency  # Тип валюты
    email: Optional[str] = None  # Email покупателя
    lang: Optional[CartLanguage] = CartLanguage.RU_RU  # Язык для сообщений об ошибках
    cart_uid: Optional[str] = None  # UID корзины
    id_po: Optional[int] = None  # Уникальный ID выбранного набора значений


class CartProductVariant(msgspec.Struct, forbid_unknown_fields=True):
    data: str
    modify: str


class CartProductOptions(msgspec.Struct, forbid_unknown_fields=True):
    id: str
    name: str
    variant: Optional[List[CartProductVariant]] = None


class CartProduct(msgspec.Struct, forbid_unknown_fields=True):
    id: int  # ID товара
    item_id: int  # ID позиции в корзине
    name: str  # Название товара
    name_original: str  # Название товара (ОРИГИНАЛЬНОЕ)
    available: bool  # Доступность для покупки (1 - доступен, 0 - недоступен)
    price: str  # Цена товара
    price_original: str  # Цена товара (ОРИГИНАЛЬНАЯ)
    currency: str  # Валюта товара
    cnt_item: int  # Количество единиц товара в корзине
    cnt_lock: bool  # Ограничение на количество (1 - ограничено, 0 - не ограничено)
    error: Any = None
    options: Optional[List[CartProductOptions]] = None


class CartAddResponse(msgspec.Struct, forbid_unknown_fields=True):
    cart_err: Optional[str]  # Код ошибки выполнения запроса
    cart_cnt: int  # Общее число товаров в корзине
    cart_uid: str  # UID корзины
    currency: str  # Основная валюта корзины
    cart_curr: List[str]  # Допустимые валюты
    products: List[CartProduct] = []  # Список товаров в корзине
    cart_err_num: Optional[Any] = None  # Дополнительный код ошибки (если есть)
    debug: Optional[Any] = None  # Отладочная информация (если есть)


# 2. Получение списка товаров корзины + обновление числа единиц товара в корзине
class CurrencyCartList(StrEnum):
    RUR = "RUR"  # Российский рубль
    USD = "USD"  # Доллар США
    EUR = "EUR"  # Евро
    UAH = "UAH"  # Украинская гривна
    MBTC = "mBTC"  # Украинская гривна


class CartListRequest(msgspec.Struct, forbid_unknown_fields=True):
    cart_uid: str  # Уникальный идентификатор корзины
    cart_curr: Optional[CurrencyCartList] = None  # Валюта для отображения цен
    item_id: Optional[int] = None  # ID товара в корзине (не путать с product_id)
    product_cnt: Optional[int] = None  # Количество единиц товара для обновления
    lang: Optional[CartLanguage] = CartLanguage.RU_RU  # Язык для отображения информации о товарах


class CartListResponse(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    cart_err: str  # Код ошибки (0, если ошибок нет)
    cart_cnt: int = None  # Общее количество товаров в корзине
    amount: str = None  # Общая сумма товаров в корзине
    currency: str = None  # Валюта для отображения
    cart_curr: List[str] = None  # Список доступных валют
    products: List[CartProduct] = None  # Список товаров в корзине с деталями (например, id, название, доступность)
    cart_err_num: str | None = None  # Дополнительный код ошибки (если есть)
    cart_err_msg: str | None = None  # Дополнительный код ошибки (если есть)
    debug: Optional[str] = None  # Отладочная информация (если есть)


# Настройка индивидуальных способов оплаты
# 1. Инициализация платежа
class InitPaymentRequest(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    invoice_id: int  # Уникальный идентификатор заказа
    amount: float  # Сумма платежа с точностью до двух знаков
    currency: str  # Валюта платежа (например, USD, RUB, EUR)
    description: str  # Описание платежа
    lang: Optional[str] = "ru-RU"  # Язык интерфейса (по умолчанию ru-RU)
    email: Optional[str] = None  # Email покупателя для расчета скидок или уведомлений
    payment_id: int  # Идентификатор выбранного метода оплаты
    return_url: str  # URL для возврата пользователя после завершения платежа


# 2. Обновление статуса платежа
class UpdatePaymentStatusRequest(msgspec.Struct, forbid_unknown_fields=True):
    invoice_id: int  # Номер заказа в системе Digiseller
    amount: float  # Сумма платежа (два знака после запятой, разделитель - точка)
    currency: str  # Валюта платежа (например, USD, RUB, EUR)
    status: str  # Текущий статус платежа (например, paid, wait, canceled)
    signature: str  # Подпись запроса для проверки подлинности
    error: Optional[str] = None  # Текст ошибки, если он присутствует


# 3. Получение статуса платежа
class PaymentStatusRequest(msgspec.Struct, forbid_unknown_fields=True):
    invoice_id: int  # Идентификатор заказа в системе
    seller_id: int  # Идентификатор продавца
    amount: str  # Сумма платежа (строка для точности)
    currency: str  # Валюта платежа (USD, RUB, EUR)
    signature: str  # Подпись запроса


class PaymentStatusResponse(msgspec.Struct, forbid_unknown_fields=True):
    invoice_id: str  # Номер заказа в системе
    amount: str  # Сумма платежа
    currency: str  # Валюта платежа
    status: str  # Статус платежа (paid, wait, canceled и т.д.)
    signature: str  # Подпись ответа для подтверждения
    error: Optional[str]  # Сообщение об ошибке, если есть
    integrator: Optional[str]  # Платежный интегратор



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct


class ProductPlatformVideoCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    url: Annotated[
        str,
        Parameter(
            title="Video URL",
            description="The URL of the video."
        )
    ]
    platform: Annotated[
        str,
        Parameter(
            title="Platform",
            description="The platform of the video."
        )
    ]


class ProductPlatformVideoUpdateRequest(Struct, forbid_unknown_fields=True):
    url: Annotated[
        str,
        Parameter(
            title="Video URL",
            description="The URL of the video."
        )
    ]
    platform: Annotated[
        str,
        Parameter(
            title="Platform",
            description="The platform of the video."
        )
    ]


class ProductPlatformVideoResponse(Struct):
    id: int
    product_platform_id: int
    version: int
    url: str
    platform: str
    created_at: datetime
    updated_at: datetime



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Enum, Index
from sqlalchemy.dialects.postgresql import BIGINT, TEXT
from sqlalchemy.orm import Mapped, relationship, mapped_column

if TYPE_CHECKING:
    from .product_platform_parameter import ProductPlatformParameter
    from .localization import Localization


class ParameterComment(BigIntAuditBase):
    __table_args__ = (
        Index("idx_parameter_comment_parameter_id", "parameter_id"),
        Index("idx_parameter_comment_localization_id", "localization_id"),
        # TODO: Think about index for both foreign keys
    )

    parameter_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product_platform_parameter.id", ondelete="CASCADE"), nullable=False)
    # locale: Mapped[Locale] = mapped_column(Enum(Locale), nullable=False)
    localization_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("localization.id", ondelete="CASCADE"), nullable=False)
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    value: Mapped[str] = mapped_column(TEXT, nullable=False)

    parameter: Mapped[ProductPlatformParameter] = relationship(
        back_populates="comments",
        foreign_keys="ParameterComment.parameter_id",
        innerjoin=True,
        uselist=False,
        lazy="noload"
    )

    locale: Mapped[Localization] = relationship(
        back_populates="parameter_comments",
        foreign_keys="ParameterComment.localization_id",
        innerjoin=True,
        uselist=False,
        lazy="joined"
    )



from __future__ import annotations

from typing import Optional, TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase, SlugKey
from sqlalchemy import ForeignKey, Index, UniqueConstraint
from sqlalchemy.dialects.postgresql import VARCHAR, BIGINT, BOOLEAN
from sqlalchemy.orm import Mapped, relationship, mapped_column, declared_attr
from sqlalchemy_utils import LtreeType

from .decorator.ltree_str import LtreeStr

if TYPE_CHECKING:
    from .product_platform_category import ProductPlatformCategory
    from .category_name import CategoryName


class Category(BigIntAuditBase):
    __table_args__ = (
        Index("idx_category_platform_id", "platform_id"),
        Index("idx_gist_path", "path", postgresql_using='gist'),
    )

    platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("platform.id", ondelete="CASCADE"), nullable=False)
    parent_id: Mapped[Optional[int]] = mapped_column(BIGINT, ForeignKey("category.id"), nullable=True)
    path: Mapped[str] = mapped_column(LtreeStr, nullable=False)
    # name: Mapped[str] = mapped_column(VARCHAR(255), nullable=False)
    external_id: Mapped[int | None] = mapped_column(BIGINT, nullable=True)
    external_parent_id: Mapped[int | None] = mapped_column(BIGINT, nullable=True)
    is_owner: Mapped[bool] = mapped_column(BOOLEAN, default=False)
    is_subcategory: Mapped[bool] = mapped_column(BOOLEAN, default=False)

    product_platform_info_categories: Mapped[list[ProductPlatformCategory]] = relationship(
        back_populates="category",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
    )

    names: Mapped[list[CategoryName]] = relationship(
        back_populates="category",
        lazy="selectin",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
    )



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct


class ProductPlatformGuaranteeCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[
        int,
        Parameter(
            title="Product Platform ID",
            description="The product platform ID."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="The is enabled."
        )
    ]
    value: Annotated[
        int,
        Parameter(
            title="Value",
            description="The value."
        )
    ]


class ProductPlatformGuaranteeUpdateRequest(Struct, forbid_unknown_fields=True):
    value: Annotated[
        int,
        Parameter(
            title="Value",
            description="The value."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="The is enabled."
        )
    ]


class ProductPlatformGuaranteeResponse(Struct):
    id: int
    product_platform_id: int
    version: int
    is_enabled: bool
    value: int
    created_at: datetime
    updated_at: datetime



from datetime import datetime
from enum import StrEnum, IntEnum
from typing import Annotated, Any, Optional, List, Union, Literal

import msgspec
from msgspec import Meta

from ..lib import BaseStruct
from ..schemas import Locale, Currency, CurrencyDigiseller, OptionTypes


# Перечисление для сортировки
class SortOrder(StrEnum):
    """Тип сортировки для списка товаров."""
    NAME = "name"  # Сортировка по названию
    NAME_DESC = "nameDESC"  # Обратная сортировка по названию
    PRICE = "price"  # Сортировка по цене
    PRICE_DESC = "priceDESC"  # Обратная сортировка по цене


class CategoriesRequest(BaseStruct, forbid_unknown_fields=True):
    """Запрос на получение списка категорий товаров."""

    # Идентификатор категории (целое число, по умолчанию 0 для выбора всего дерева каталога)
    category_id: Annotated[
        int,
        Meta(
            description="Идентификатор категории (целое число, по умолчанию 0 для выбора всего дерева каталога)"
        )
    ] = 0  # По умолчанию выбирается все дерево каталога

    # Язык отображения информации (по умолчанию ru-RU)
    lang: Annotated[
        Locale,
        Meta(description="Язык отображения информации (по умолчанию ru-RU)")
    ] = Locale.RU_RU


class Subcategory(msgspec.Struct, forbid_unknown_fields=True):
    id: int  # Идентификатор подкатегории (целое число)
    name: str  # Название подкатегории (строка)
    cnt: int  # Количество товаров в подкатегории (целое число)


class Category(msgspec.Struct, forbid_unknown_fields=True):
    id: str  # Идентификатор категории (целое число)
    name: str  # Название категории (строка)
    cnt: str  # Количество товаров в категории (целое число)
    sub: Optional[List[Subcategory]] = None


class CategoriesResponse(msgspec.Struct, forbid_unknown_fields=True):
    retval: int  # Код выполнения запроса (целое число)
    retdesc: Optional[str] = None  # Расшифровка кода выполнения запроса (строка)
    category: List[Category] = None  # Список категорий магазина (список объектов Category)
    queryId: Any = None  # я хз что это


# Запрос на получение товаров из категории
class ProductsRequest(BaseStruct, forbid_unknown_fields=True, kw_only=True):
    """Запрос на получение списка товаров в категории."""

    # Идентификатор категории (обязательное поле)
    category_id: Annotated[
        int,
        Meta(
            ge=-3,  # Минимальное значение -3 (популярные товары)
            le=0,  # Максимальное значение 0 (товары на главной странице)
            description="""
                Идентификатор категории:
                0 - товары на главной странице,
                -1 - товары со знаком 'скидка',
                -2 - товары со знаком 'новинка',
                -3 - товары со знаком 'популярный'
            """
        )
    ] = 0  # По умолчанию - товары на главной странице

    # Номер страницы (по умолчанию 1)
    page: Annotated[
        int,
        Meta(
            ge=1,  # Минимальное значение 1
            description="Номер страницы (по умолчанию 1)"
        )
    ] = 1

    # Количество строк на странице (по умолчанию 20, максимум 500)
    rows: Annotated[
        int,
        Meta(
            ge=20,  # Минимальное значение 20
            le=500,  # Максимальное значение 500
            description="Количество строк на странице (по умолчанию 20, максимум 500)"
        )
    ] = 20

    # Способ сортировки товаров (необязательное поле)
    order: Union[SortOrder, msgspec.UNSET] = msgspec.UNSET

    # Валюта отображения стоимости товаров (необязательное поле)
    currency: Annotated[
        Optional[Currency],
        Meta(description="Тип валюты для отображения стоимости товара: USD, RUB, EUR, UAH")
    ] = Currency.RUB

    # Язык отображения информации (по умолчанию ru-RU)
    lang: Annotated[
        Optional[Locale],
        Meta(description="Язык отображения информации (по умолчанию ru-RU)")
    ] = Locale.RU_RU


# Данные о распродаже товара
class SaleInfo(msgspec.Struct, forbid_unknown_fields=True):
    """Информация о распродаже товара."""
    common_base_price: Optional[float] = None  # Базовая стоимость товара до распродажи
    common_price_usd: Optional[float] = None  # Цена до распродажи в долларах США
    common_price_rur: Optional[float] = None  # Цена до распродажи в рублях
    common_price_eur: Optional[float] = None  # Цена до распродажи в евро
    common_price_uah: Optional[float] = None  # Цена до распродажи в гривнах
    sale_begin: Optional[float] = None  # Дата начала распродажи
    sale_end: Optional[float] = None  # Дата окончания распродажи
    sale_percent: Optional[float] = None  # Процент скидки на товар


# Ответ по каждому товару в категории
class ProductResponse(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    """Ответ с информацией о товаре в категории."""
    id: str  # Идентификатор товара
    name: str  # Название товара
    cntImg: Optional[int]  # Количество изображений товара (необязательное поле)
    info: Optional[str] = None  # Описание товара (необязательное поле)
    price: str  # Стоимость товара
    base_price: str  # Цена в базовой валюте, установленной продавцом
    base_currency: CurrencyDigiseller  # Базовая валюта, в которой установлена цена товара
    currency: Currency  # Валюта отображения стоимости товара
    price_rub: str  # Стоимость товара в рублях
    price_usd: str  # Стоимость товара в долларах США
    price_eur: str  # Стоимость товара в евро
    price_uah: str  # Стоимость товара в гривнах
    partner_comiss: str  # Партнерская комиссия (в процентах)
    agency_id: str  # Идентификатор партнера
    collection: Optional[str] = None  # Тип содержимого товара (необязательное поле)
    is_available: int  # Наличие товара (0 - отсутствует, 1 - в наличии)
    has_discount: int  # Скидка постоянным покупателям (0 - нет скидки, 1 - есть скидка)
    id_present: int  # ID товара-подарка (0 - подарок не задан)
    sale_info: Optional[SaleInfo] = None  # Информация о распродаже товара (необязательное поле)
    label: Optional[str] = None  # Метка товара (скидка, новинка и т.д., необязательное поле)


# Полный ответ на запрос списка товаров в категории
class ProductsResponse(msgspec.Struct, forbid_unknown_fields=True):
    """Ответ на запрос получения списка товаров в категории."""
    retval: int  # Код выполнения запроса (0 - запрос выполнен)
    retdesc: Optional[str] = None  # Расшифровка кода выполнения запроса
    lang: Optional[str] = None  # Язык отображения информации
    total_pages: int = msgspec.field(name='totalPages', default=None)  # Общее количество страниц
    total_items: int = msgspec.field(name='totalItems', default=None)  # Общее количество товаров
    bread_crumbs: List[dict] = msgspec.field(name='breadCrumbs',
                                             default=None)  # Хлебные крошки - путь к выбранной категории
    categories: List[dict] = None  # Список подкатегорий
    product: List[ProductResponse] = None  # Список товаров в категории
    queryId: Any = None  # Ну не ебу я что это


class ProductsDescriptionRequest(BaseStruct, forbid_unknown_fields=True):
    """Схема запроса для получения описаний товаров по их ID."""
    ids: List[int]  # Идентификаторы товаров (максимум 2000)
    lang: Locale = Locale.RU_RU  # Язык отображения информации, по умолчанию 'ru-RU'


class ProductDescription(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    """Ответ с информацией о каждом товаре."""
    id: int  # Идентификатор товара
    id_seller: int  # Идентификатор продавца
    name: str  # Название товара
    info: Optional[str] = ""  # Описание товара (опционально)
    add_info: Optional[str] = ""  # Дополнительная информация о товаре (опционально)
    collection: Optional[str] = ""  # Вид содержимого товара (digi, pins, unit, book, soft)
    base_currency: CurrencyDigiseller  # Базовая валюта
    base_price: float  # Цена в базовой валюте, установленной продавцом
    price_usd: float  # Цена в USD
    price_rub: float  # Цена в RUB
    price_eur: float  # Цена в EUR
    price_uah: float  # Цена в UAH
    cnt_sell: int  # Количество продаж (-1 означает полностью скрыто)
    cnt_sell_hidden: int  # Режим отображения числа продаж (0 - продажи открыты, 1 - скрыты)
    cnt_return: int  # Количество возвратов (-1 означает полностью скрыто)
    cnt_return_hidden: int  # Режим отображения возвратов (0 - возвраты открыты, 1 - скрыты)
    in_stock: int  # Доступность товара (0 - товар отсутствует, 1 - в наличии)
    um_in_stock: Optional[int] = None  # Доступное количество на складе
    num_in_lock: Optional[int] = None  # Количество товаров, зарезервированных под оплату
    owner_id: int  # Код торговой площадки
    release_date: Optional[datetime] = None  # Дата релиза (может быть null)
    sale_info: Optional[SaleInfo] = None  # Информация о распродаже (опционально)


class ProductsDescriptionResponse(msgspec.Struct, forbid_unknown_fields=True):
    """Ответ с информацией о нескольких товарах."""
    products: List[ProductDescription]  # Список товаров


class ProductDescriptionRequest(BaseStruct, forbid_unknown_fields=True):
    """Схема запроса для получения описания товара по его ID."""
    product_id: int  # Идентификатор товара
    partner_uid: Optional[str] = None  # Партнерский UID для учета индивидуальных отчислений
    currency: Optional[Currency] = Currency.RUB  # Валюта отображения цены товара (по умолчанию USD)
    lang: Optional[Locale] = Locale.RU_RU  # Язык отображения информации (по умолчанию ru-RU)
    owner: Optional[int] = None  # Признак отображения всех способов оплаты (1 - показать)
    showHiddenVariants: Optional[int] = None  # Признак отображения всех вариантов параметров (1 - показать)


class CurrencyPrice(msgspec.Struct, forbid_unknown_fields=True):
    """Содержит цены в разных валютах."""
    RUB: float  # Цена в рублях
    USD: float  # Цена в долларах США
    EUR: float  # Цена в евро
    UAH: float  # Цена в гривнах
    mBTC: Optional[float] = None  # Цена в mBTC (миллибиткойн), если доступно
    mLTC: Optional[float] = None  # Цена в mLTC (миллилиткойн), если доступно


class PaymentLimit(msgspec.Struct, forbid_unknown_fields=True):
    """Ограничения способа оплаты."""
    min: Optional[float] = None  # Минимальная сумма платежа
    max: Optional[float] = None  # Максимальная сумма платежа
    currency: Optional[str] = None  # Валюта, в которой действует ограничение


class PaymentCurrency(msgspec.Struct, forbid_unknown_fields=True):
    """Информация о валюте и цене для определенного способа оплаты."""
    currency: str  # Валюта платежа
    code: str  # Код способа оплаты
    price: Optional[float] = None  # Цена в указанной валюте
    limit: Optional[PaymentLimit] = None  # Ограничения на сумму платежа


class PaymentMethod(msgspec.Struct, forbid_unknown_fields=True):
    """Информация о способе оплаты."""
    name: str  # Название способа оплаты
    currencies: List[PaymentCurrency]  # Список валют для этого способа оплаты
    code: Optional[str] = None  # Код метода оплаты (его не было)
    hide: Optional[int] = None  # Параметр для скрытия метода оплаты (его не было)


class PricesUnit(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    """Информация о цене и единицах товара."""
    unit_name: str  # Название единицы товара
    unit_amount: float  # Количество единиц товара
    unit_amount_desc: str  # Описание единицы товара
    unit_currency: str  # Валюта для единицы товара
    unit_cnt: float  # Количество единиц товара
    unit_cnt_min: Optional[int | float] = None  # Минимальное количество единиц, которое можно купить
    unit_cnt_max: Optional[int | float] = None  # Максимальное количество единиц, которое можно купить
    unit_cnt_desc: Optional[str]  # Описание количества единиц товара
    unit_fixed: Optional[List[int]] = None  # Фиксированные значения количества единиц
    unit_only_int: Optional[int] = None  # Только целое количество (не дробное)


class PreviewImage(msgspec.Struct, forbid_unknown_fields=True):
    """Предварительное изображение товара."""
    id: int  # Идентификатор изображения
    url: str  # URL изображения
    width: int  # Ширина изображения в пикселях
    height: int  # Высота изображения в пикселях


class PreviewVideoType(StrEnum):
    """Типы предварительного видео."""
    VIDEO = "video"
    YOUTUBE = "youtube"
    VIMEO = "vimeo"


class PreviewVideo(msgspec.Struct, forbid_unknown_fields=True):
    """Предварительное видео товара."""
    type: PreviewVideoType  # Тип видео (youtube, vimeo, etc.)
    id: str  # Идентификатор видео
    preview: str  # URL предварительного изображения для видео


class TextProduct(msgspec.Struct, forbid_unknown_fields=True):
    """Информация о продукте в текстовом формате."""
    date: datetime  # Дата размещения текста
    size: int  # Количество символов текста


class FileProduct(msgspec.Struct, forbid_unknown_fields=True):
    """Информация о продукте в формате файла."""
    date: datetime  # Дата размещения файла
    size: int  # Размер файла в байтах
    name: str  # Имя файла
    trial: Optional[int] = None  # Пробная версия (если доступно)


class Breadcrumb(msgspec.Struct, forbid_unknown_fields=True):
    """Хлебные крошки для навигации по категориям."""
    id: int  # Идентификатор категории
    name: str  # Название категории
    products_cnt: int  # Количество товаров в категории


class Discount(msgspec.Struct, forbid_unknown_fields=True):
    """Информация о скидке на товар."""
    summa: float  # Пороговое значение для скидки
    percent: float  # Процент скидки


class UnitDiscount(msgspec.Struct, forbid_unknown_fields=True):
    """Скидка на товар, рассчитанная на единицу."""
    desc: str  # Описание скидки
    percent: float  # Процент скидки
    price: float  # Цена с учетом скидки
    cnt: float | None = None  # Количество товаров для скидки


class Units(msgspec.Struct, forbid_unknown_fields=True):
    """Информация о товаре с переменной ценой или моделью 'плати сколько хочешь'."""
    desc: str  # Описание единицы товара
    price: float  # Цена за единицу товара
    discounts: Optional[List[UnitDiscount]] = None  # Список скидок на товар


class Present(msgspec.Struct, forbid_unknown_fields=True):
    """Информация о подарке."""
    id: int  # Идентификатор подарка
    name: str  # Название подарка


class OptionVariantModifyType(StrEnum):
    PERCENT = "%"
    USD = "USD"
    RUB = "RUB"
    EUR = "EUR"
    UAH = "UAH"


# , forbid_unknown_fields=True
class OptionVariant(msgspec.Struct, kw_only=True):
    """Вариант значения параметра товара."""
    value: int  # Значение варианта
    text: str  # Название варианта
    default: Optional[int] = None  # Значение по умолчанию
    modify: Optional[str] = None  # Изменение цены товара при выборе варианта
    modify_type: Optional[OptionVariantModifyType | Any] = None  # Тип изменения цены (% или фиксированное значение)
    modify_value: Optional[float] = None  # Значение изменения цены
    modify_value_default: Optional[float] = None  # Значение изменения цены по умолчанию
    num_in_stock: Optional[int] = None  # Количество товара на складе
    visible: int  # Параметр видимости 1 - не скрыт иначе - скрыт


# forbid_unknown_fields=True
class Option(msgspec.Struct, kw_only=True):
    """Параметры, определяемые покупателем при оплате."""
    name: str  # Имя параметра
    label: str  # Метка параметра
    comment: str  # Описание параметра
    type: OptionTypes  # Тип параметра (checkbox, text, radio, etc.)
    separate_content: int  # Признак наличия раздельного содержимого
    required: int  # Обязателен ли параметр
    modifier_visible: Optional[int] = None  # Показ модификатора цены
    variants: Optional[List[OptionVariant]] = None  # Список вариантов значений


class Statistics(msgspec.Struct, forbid_unknown_fields=True):
    """Статистика по продажам товара."""
    sales: int  # Количество продаж
    sales_hidden: int  # Параметр скрытия количества продаж
    refunds: int  # Количество возвратов
    refunds_hidden: int  # Параметр скрытия количества возвратов
    good_reviews: int  # Количество положительных отзывов
    good_reviews_hidden: int  # Параметр скрытия положительных отзывов
    bad_reviews: int  # Количество отрицательных отзывов
    bad_reviews_hidden: int  # Параметр скрытия отрицательных отзывов


class Seller(msgspec.Struct, forbid_unknown_fields=True):
    """Информация о продавце товара."""
    id: int  # Идентификатор продавца
    name: str  # Имя продавца


class Collection(StrEnum):
    """Тип коллекции товара."""
    BOOK = "book"  # Электронная книга
    DIGI = "digi"  # Цифровой товар
    PINS = "pins"  # Пин-коды
    SOFT = "soft"  # Программа
    UNIT = "unit"  # Товар с нефиксированной ценой


class ProductType(StrEnum):
    TEXT = "text"
    FILE = "file"


class Prices(msgspec.Struct, forbid_unknown_fields=True):
    """Цены товара."""
    initial: CurrencyPrice  # Базовая цена товара
    default: CurrencyPrice  # Цена основного способа оплаты


class Product(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    """Информация о товаре."""
    id: int  # Идентификатор товара
    id_prev: int  # Идентификатор предыдущего товара в категории
    id_next: int  # Идентификатор следующего товара в категории
    name: str  # Название товара
    price: float = None  # Цена товара
    currency: Currency  # Валюта товара
    url: Optional[str]  # URL товара
    info: Optional[str] = None  # Описание товара
    add_info: Optional[str] = None  # Дополнительная информация о товаре
    release_date: Optional[str] = None  # Дата релиза товара
    agency_fee: float  # Агентское вознаграждение
    agency_sum: float  # Сумма агентского вознаграждения
    agency_id: Optional[int] = None  # Идентификатор агента
    collection: Collection  # Коллекция товара
    property_good: Annotated[
        int, Meta(ge=1, le=2, description="1 - универсальное, 2 - уникальное")
    ] = msgspec.field(name='propertygood')  # Тип содержимого товара
    is_available: Annotated[
        int, Meta(ge=0, le=1, description="0 - товар отсутствует, 1 - товар в наличии")
    ]  # Доступность товара
    show_rest: Annotated[
        int, Meta(ge=0, le=1, description="0 - скрыт, 1 - отображается остаток")
    ]  # Отображение остатка

    num_in_stock: Optional[int] = None  # Доступное количество товара на складе
    num_in_lock: int  # Количество товаров, зарезервированных под оплату
    pwyw: Optional[int] = None  # Параметр "плати сколько хочешь"
    label: Optional[str] = None  # Метка товара
    prices: Optional[Prices] = None  # Цены товара

    payment_methods: List[PaymentMethod]  # Способы оплаты
    prices_unit: Optional[PricesUnit] = None  # Цены за единицу товара
    preview_imgs: Optional[List[PreviewImage]] = None  # Изображения товара для предпросмотра
    preview_videos: Optional[List[PreviewVideo]] = None  # Видео товара для предпросмотра

    type: ProductType  # Тип товара
    text: Optional[TextProduct] = None  # Параметры для текстовых товаров
    file: Optional[FileProduct] = None  # Параметры для файловых товаров
    category_id: int  # Идентификатор категории товара
    breadcrumbs: List[Breadcrumb]  # Навигация по категориям
    discounts: Optional[List[Discount]] = None  # Скидки на товар
    units: Optional[Units] = None  # Сведения о единицах товара

    present: Optional[Present] = None  # Информация о подарке
    gift_commiss: Optional[float] = None  # Комиссия на подарок
    options: Optional[List[Option]] = None  # Параметры товара для выбора при оплате
    options_check: Annotated[
        int,
        Meta(ge=0, le=1,
             description="""
                индикатор включенной опции 
                "перед оплатой отправлять параметры на мой сервер для проверки" 
                (определяется в Параметрах товара)
                """
             )
    ]  # Параметр проверки опций перед оплатой
    statistics: Statistics  # Статистика по товару
    seller: Seller  # Информация о продавце товара
    sale_info: Optional[SaleInfo] = None  # Информация о распродаже

    # Хуй ебу что это
    owner: Any = None
    section_id: Any = None  # айди подкатегории плати маркета
    no_cart: Any = None
    type_good: Any = None
    # discounts: Any = None
    # options: Any = None


class ProductDescriptionResponse(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    """Ответ API с полной информацией о товаре."""
    retval: int  # Код выполнения запроса
    retdesc: Optional[str] = None  # Описание кода выполнения запроса
    product: Product = None  # Описание товара
    queryId: Optional[Any] = None  # Идентификатор запроса, если доступен


class ProductPriceRequest(BaseStruct, forbid_unknown_fields=True):
    """Схема запроса для получения цены товара с учетом различных параметров."""

    # Идентификатор товара, который необходимо передать для получения цены.
    product_id: Annotated[
        int,
        Meta(
            description="Идентификатор товара"
        )
    ]
    # Опциональный список строк, каждая строка представляет собой пару optionId:valueId.
    # Например, options[]=28532:41541 для передачи значимых параметров, влияющих на цену товара.
    # "options[]": ["28532:41541", "28530:41534"]
    options: Annotated[
        List[str],
        Meta(
            description="Перечисление пар optionId:valueId, например, options[]=28532:41541&options[]=28530:41534"
        )
    ]

    # Опциональная строка, представляющая код валюты. Например, WMR, WMZ и другие трехзначные коды валют.
    currency: Annotated[
        str,
        Meta(
            description="Валюта, трехзначный код метода оплаты (WMR | WMZ | ... )"
        )
    ]

    # Опциональное поле для указания стоимости, которая рассчитывается исходя из указанного количества единиц товара.
    amount: Annotated[
        float,
        Meta(
            description="Стоимость, исходя из указанного количества единиц товара"
        )
    ] = msgspec.UNSET

    # Опциональное поле для указания количества товара с нефиксированной ценой.
    # Обязательный параметр для товаров с нефиксированной ценой.
    unit_cnt: Annotated[
        Optional[int],
        Meta(
            description="Количество товара с нефиксированной ценой, обязателен для товара с нефиксированной ценой"
        )
    ] = msgspec.UNSET

    # Опциональное поле для указания количества товара с фиксированной ценой.
    count: Annotated[
        Optional[int],
        Meta(
            description="Количество товара с фиксированной ценой"
        )
    ] = msgspec.UNSET


class ProductPriceData(msgspec.Struct, forbid_unknown_fields=True):
    """Данные о цене товара."""
    price: Optional[float] = None  # Цена товара в базовой валюте
    count: Optional[float] = None  # Количество единиц товара, исходя из указанной стоимости
    amount: Optional[float] = None  # Стоимость, исходя из указанного количества единиц товара
    currency: Optional[str] = None  # Тип валюты
    commission: Optional[float] = None  # Комиссия в зависимости от способа оплаты
    free_pay: Any = None
    sale_info: Optional[SaleInfo] = None  # Информация о распродаже


class ProductPriceResponse(msgspec.Struct, forbid_unknown_fields=True):
    """Ответ API с информацией о цене товара."""
    retval: int  # Код выполнения запроса (0 - успех, 1 и другие - ограничения)
    retdesc: Optional[str] = None  # Описание кода выполнения запроса
    data: Optional[ProductPriceData] = None  # Данные о цене товара


class ReviewType(StrEnum):
    GOOD = "good"
    BAD = "bad"
    ALL = "all"


class OwnerID(IntEnum):
    OWN_SHOP = 0  # собственный магазин
    PLATI_MARKET = 1  # plati.market
    GGESEL = 1271  # ggesel
    WMCENTER = 9295  # wmcenter.net


class ProductReviewsRequest(BaseStruct, forbid_unknown_fields=True, kw_only=True):
    product_id: Optional[int] = msgspec.UNSET  # Идентификатор товара (необязательный параметр)
    type: ReviewType  # Тип отзыва: good, bad, all
    owner_id: Optional[OwnerID] = OwnerID.OWN_SHOP  # Идентификатор торговой площадки
    page: Optional[int] = 1  # Номер страницы (по умолчанию 1)
    rows: Optional[int] = 20  # Количество строк на странице (по умолчанию 20)
    lang: Optional[Locale] = Locale.RU_RU  # Язык отображения информации (по умолчанию ru-RU)


class ProductReview(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    id: int  # Идентификатор отзыва
    invoice_id: int  # Номер заказа
    owner_id: int  # Идентификатор торговой площадки
    type: str  # Тип отзыва: good или bad
    good: Optional[int] = None  # Идентификатор товара (если не был передан в запросе)
    name: Optional[str] = None  # Название товара (если не было передано в запросе)
    date: datetime  # Дата отзыва
    info: str  # Содержание отзыва
    comment: Optional[str] = None  # Комментарий продавца
    dateUtc: datetime


class ProductReviewsResponse(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    retval: int  # Код выполнения запроса (0 - успех, 1 и другие - ограничения)
    retdesc: Optional[str] = None  # Описание кода выполнения запроса
    total_pages: Optional[int] = msgspec.field(name='totalPages', default=None)  # Всего страниц
    total_items: Optional[int] = msgspec.field(name='totalItems', default=None)  # Всего отзывов
    total_good: Optional[int] = msgspec.field(name='totalGood', default=None)  # Всего положительных отзывов
    total_bad: Optional[int] = msgspec.field(name='totalBad', default=None)  # Всего отрицательных отзывов
    reviews: List[ProductReview] = msgspec.field(name='review', default=None)  # Список отзывов


class OrderCol(StrEnum):
    NAME = "name"
    PRICE = "price"
    CNT_SELL = "cntsell"
    CNT_RETURN = "cntreturn"
    CNT_GOOD_RESPONSES = "cntgoodresponses"
    CNT_BAD_RESPONSES = "cntbadresponses"


class OrderDir(StrEnum):
    ASC = "asc"
    DESC = "desc"


class ShowHidden(IntEnum):
    WITHOUT_HIDDEN = 0  # Без скрытых товаров
    WITH_HIDDEN = 1  # Со скрытыми товарами
    ONLY_HIDDEN = 2  # Только скрытые товары


class LocaleSellerProducts(StrEnum):
    """Язык отображения информации."""
    RU_RU = "ru-RU"
    EN_US = "en-US"


class SellerProductsRequest(BaseStruct, forbid_unknown_fields=True):
    order_col: OrderCol = OrderCol.CNT_SELL  # Поле сортировки (по умолчанию - количество продаж)
    order_dir: OrderDir = OrderDir.DESC  # Порядок сортировки (по умолчанию - по убыванию)
    rows: Annotated[
        int,
        Meta(ge=10, le=1000)
    ] = 10  # Количество на странице (по умолчанию 10, не более 1000)
    page: Annotated[
        int,
        Meta(ge=1)
    ] = 1  # Номер страницы (по умолчанию 1)
    currency: Optional[Currency] = Currency.RUB  # Тип валюты для отображения цены товара
    lang: Optional[
        LocaleSellerProducts] = LocaleSellerProducts.RU_RU  # Язык отображения информации (по умолчанию ru-RU)
    show_hidden: Optional[
        ShowHidden] = ShowHidden.WITHOUT_HIDDEN  # Скрытые товары (0 - без скрытых, 1 - со скрытыми, 2 - только скрытые)
    owner_id: Optional[OwnerID] = msgspec.UNSET  # Идентификатор торговой площадки (опционально)


class ProductRow(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    id_goods: int
    name_goods: str
    info_goods: Optional[str] = None
    add_info: Optional[str] = None
    price: float
    currency: Currency  # Сделать выдачу Currency
    cnt_sell: int
    cnt_sell_hidden: int
    cnt_return: int
    cnt_return_hidden: int
    cnt_goodresponses: int
    cnt_goodresponses_hidden: int
    cnt_badresponses: int
    cnt_badresponses_hidden: int
    price_usd: float
    price_rur: float
    price_eur: float
    price_uah: float
    in_stock: int
    num_in_stock: int
    visible: int
    commiss_agent: float
    has_discount: bool
    num_options: int
    sale_info: Optional[SaleInfo] = None
    owner_id: int
    release_date: Optional[datetime] = None
    num_in_stock: Optional[int] = None


class SellerProductsResponse(msgspec.Struct, forbid_unknown_fields=True, kw_only=True):
    retval: int
    retdesc: Optional[str] = None
    id_seller: Optional[int] = None
    name_seller: Optional[str] = None
    cnt_goods: Optional[int] = None
    pages: Optional[int] = None
    page: Optional[int] = None
    order_col: Optional[OrderCol] = None
    order_dir: Optional[OrderDir] = None
    rows: Optional[List[ProductRow]] = None
    rating_seller: Optional[float] = None
    show_hidden: Optional[float] = None


class CurrencyProductDiscount(StrEnum):
    RUB = "RUB"  # Российский рубль
    RUR = "RUR"  # Российский рубль

    USD = "USD"  # Доллар США
    EUR = "EUR"  # Евро
    UAH = "UAH"  # Украинская гривна
    MBTC = "mBTC"


class ProductDiscountRequest(msgspec.Struct, forbid_unknown_fields=True):
    product_id: int  # Идентификатор товара (целое число)
    currency: CurrencyProductDiscount  # Тип валюты (USD, RUR, EUR, UAH, mBTC)
    email: str  # Email покупателя


class ProductDiscountInfo(msgspec.Struct, forbid_unknown_fields=True):
    price: float  # Цена с учетом скидки
    currency: CurrencyProductDiscount  # Валюта товара


class DiscountInfo(msgspec.Struct, forbid_unknown_fields=True):
    percent: float  # Процент скидки
    total: float  # Общая сумма покупок (всегда в USD)
    currency: Literal['USD']  # Валюта скидки (всегда USD)


class ProductDiscountResponse(msgspec.Struct, forbid_unknown_fields=True):
    retval: int  # Код выполнения запроса (0 - успех)
    retdesc: Optional[str] = None  # Расшифровка кода выполнения запроса
    product: Optional[ProductDiscountInfo] = None  # Информация о товаре
    discount: Optional[DiscountInfo] = None  # Информация о скидке


class ProductSearchRequest(msgspec.Struct, forbid_unknown_fields=True):
    # Строка поиска
    search: Annotated[
        Optional[str],
        Meta(description="Строка поиска")
    ] = None
    currency: Annotated[
        Optional[Currency],
        Meta(description="Тип валюты для отображения стоимости товара: USD, RUB, EUR, UAH")
    ] = Currency.RUB
    # Номер страницы (по умолчанию 1)
    page: Annotated[
        int,
        Meta(
            ge=1,  # Минимальное значение 1
            description="Номер страницы (по умолчанию 1)"
        )
    ] = 1
    # Количество строк на странице (по умолчанию 20, максимум 500)
    rows: Annotated[
        int,
        Meta(
            ge=20,  # Минимальное значение 20
            le=500,  # Максимальное значение 500
            description="Количество строк на странице (по умолчанию 20, максимум 500)"
        )
    ] = 20

    # Язык отображения информации (по умолчанию ru-RU)
    lang: Annotated[
        Optional[Locale],
        Meta(description="Язык отображения информации (по умолчанию ru-RU)")
    ] = Locale.RU_RU


class Snippet(msgspec.Struct, forbid_unknown_fields=True):
    name: Optional[str] = None  # Название товара с найденной строкой поиска
    info: Optional[str] = None  # Фрагмент(ы) описания товара с найденной строкой поиска


class ProductSearchInfo(msgspec.Struct, forbid_unknown_fields=True):
    id: int  # Идентификатор товара
    name: str  # Название товара
    price: float  # Стоимость товара
    agency_fee: Optional[float] = None  # Партнерское вознаграждение %
    snippets: Optional[Snippet] = None  # Информация о товаре с найденной строкой поиска
    sale_info: Optional[SaleInfo] = None  # Информация о распродаже


class PageInfo(msgspec.Struct, forbid_unknown_fields=True):
    num: int  # Номер страницы, переданный в запросе
    rows: int  # Количество строк на странице
    cnt: int  # Количество страниц


class ProductSearchResponse(msgspec.Struct, forbid_unknown_fields=True):
    retval: int  # Код выполнения запроса (0 - успех)
    retdesc: Optional[str] = None  # Расшифровка кода выполнения запроса
    pages: Optional[PageInfo] = None  # Информация о страницах
    products: Optional[List[ProductSearchInfo]] = None  # Список продуктов
    search: Optional[str] = None  # Строка поиска, переданная в запросе
    currency: Optional[str] = None  # Тип валюты для отображения цены, переданный в запросе


class ProductImageRequest(BaseStruct, forbid_unknown_fields=True):
    # Идентификатор товара
    id_d: Annotated[
        int,
        Meta(description="Идентификатор товара (целое число)")
    ]

    # Максимальная длина в пикселях по бОльшей стороне
    maxlength: Annotated[
        Optional[int],
        Meta(description="Максимальная длина в пикселях по бОльшей стороне")
    ] = msgspec.UNSET

    # Максимальная ширина изображения
    w: Annotated[
        Optional[int],
        Meta(description="Максимальная ширина изображения (целое число)")
    ] = msgspec.UNSET

    # Максимальная высота изображения
    h: Annotated[
        Optional[int],
        Meta(description="Максимальная высота изображения (целое число)")
    ] = msgspec.UNSET

    # Обрезание для образования квадрата
    crop: Annotated[
        Optional[bool],
        Meta(description="Обрезание бОльших сторон прямоугольника для образования квадрата (true | false)")
    ] = msgspec.UNSET


class ProductImageResponse(msgspec.Struct, forbid_unknown_fields=True):
    image_data: Optional[bytes] = None  # Данные изображения в бинарном виде


class CloneProductRequest(BaseStruct, forbid_unknown_fields=True):
    product_id: int  # Идентификатор товара
    count: Annotated[int, Meta(ge=1, le=5)] = 1  # Количество создаваемых копий (по умолчанию: 1, не более 5)
    categories: Optional[bool] = False  # Разместить копии в тех же категориях (по умолчанию: false)
    notify: Optional[bool] = False  # Копировать настройки уведомлений (по умолчанию: false)
    discounts: Optional[bool] = False  # Копировать настройки скидок (по умолчанию: false)
    options: Optional[bool] = False  # Копировать настройки дополнительных параметров (по умолчанию: false)
    comissions: Optional[bool] = False  # Копировать настройки компенсаций (по умолчанию: false)
    gallery: Optional[bool] = False  # Копировать изображения и видео (по умолчанию: false)


class ErrorInfo(msgspec.Struct, forbid_unknown_fields=True):
    code: str  # Код ошибки
    message: str  # Описание кода выполнения


class Content(msgspec.Struct, forbid_unknown_fields=True):
    products: List[int]  # Идентификаторы созданных товаров


class CloneProductResponse(msgspec.Struct, forbid_unknown_fields=True):
    retval: int  # Код выполнения запроса (0 - успех, 1 - ошибка)
    retdesc: Optional[str] = None  # Описание кода выполнения
    errors: Optional[List[ErrorInfo]] = None  # Массив ошибок, если есть
    content: Optional[Content] = None  # Тело ответа (если успешно)


class SellerOfferProductsRequest(BaseStruct, forbid_unknown_fields=True):
    # Название товара (необязательное поле)
    productName: Annotated[
        Optional[str],
        Meta(description="Название товара (строка)")
    ] = msgspec.UNSET

    # Идентификатор товара (необязательное поле)
    productId: Annotated[
        Optional[int],
        Meta(description="Идентификатор товара (целое число)")
    ] = msgspec.UNSET

    # Только товары в наличии
    onlyInStock: Annotated[
        Optional[bool],
        Meta(description="Только товары в наличии (по умолчанию: false)")
    ] = False

    # Только с индивидуальными комиссионными
    onlyIndividual: Annotated[
        Optional[bool],
        Meta(description="Только с индивидуальными комиссионными (по умолчанию: false)")
    ] = False

    # Номер страницы
    page: Annotated[
        int,
        Meta(ge=1, description="Номер страницы (целое число)")
    ] = 1

    # Количество элементов на странице (1 - 100)
    count: Annotated[
        int,
        Meta(ge=1, le=100, description="Количество элементов на странице (целое число, 1 - 100)")
    ] = 20


class ProductName(msgspec.Struct, forbid_unknown_fields=True):
    locale: str  # Локаль (например, 'ru-RU' или 'en-US')
    value: str  # Название товара на указанной локали


class ProductItem(msgspec.Struct, forbid_unknown_fields=True):
    product_id: int  # Идентификатор товара
    name: List[ProductName]  # Название товара
    price: float  # Цена
    currency: Currency  # Валюта (RUB, USD, UAH, EUR)
    in_affiliate: bool  # Товар участвует в партнерской программе
    individual_percent: int  # Индивидуальный процент отчислений
    global_percent: int  # Процент отчислений в настройках товара
    last_change: Optional[str] = None  # Последние изменения (ISO 8601)


class ContentInfo(msgspec.Struct, forbid_unknown_fields=True):
    page: int  # Номер страницы
    count: int  # Количество на странице
    total_count: int  # Количество всех элементов
    total_pages: int  # Всего страниц
    has_next_page: bool  # Признак наличия следующей страницы
    has_previous_page: bool  # Признак наличия предыдущей страницы
    items: List[ProductItem]  # Список товаров


class SellerOfferProductsResponse(msgspec.Struct, forbid_unknown_fields=True):
    retval: int  # Код выполнения запроса (0 - успех, 1 - ошибка)
    retdesc: Optional[str] = None  # Описание кода выполнения запроса
    errors: Optional[List[ErrorInfo]] = None  # Массив ошибок (если есть)
    content: Optional[ContentInfo] = None  # Тело ответа (если нет ошибок)



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Enum, Index
from sqlalchemy.dialects.postgresql import BIGINT, BOOLEAN, INTEGER, FLOAT
from sqlalchemy.orm import Mapped, relationship, mapped_column

from .enums import ModifierOperator


if TYPE_CHECKING:
    from .product_platform_parameter import ProductPlatformParameter
    from .parameter_option_name import ParameterOptionName


class ParameterOption(BigIntAuditBase):
    __table_args__ = (
        Index("idx_parameter_option_parameter_id", "parameter_id"),
    )

    parameter_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product_platform_parameter.id", ondelete="CASCADE"), nullable=False)
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    modifier_operator: Mapped[ModifierOperator] = mapped_column(Enum(ModifierOperator, name="modifier_operator_enum"), nullable=False)
    modifier_value: Mapped[float] = mapped_column(FLOAT, nullable=False)
    is_default: Mapped[bool] = mapped_column(BOOLEAN, default=False)
    is_enabled: Mapped[bool] = mapped_column(BOOLEAN, default=True)
    order: Mapped[int] = mapped_column(INTEGER, nullable=False)

    parameter: Mapped[ProductPlatformParameter] = relationship(
        back_populates="options",
        foreign_keys="ParameterOption.parameter_id",
        innerjoin=True,
        uselist=False,
        lazy="noload"
    )

    names: Mapped[list[ParameterOptionName]] = relationship(
        back_populates="parameter_option",
        lazy="selectin",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ParameterOptionName.id)"
    )



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct


class OriginPlatformCreateRequest(Struct, forbid_unknown_fields=True):
    name: Annotated[
        str,
        Parameter(
            title="Название платформы",
            description="The name of the origin platform."
        )
    ]
    slug: Annotated[
        str,
        Parameter(
            title="Slug",
            description="A human-readable identifier for the platform."
        )
    ]


class OriginPlatformUpdateRequest(Struct, forbid_unknown_fields=True):
    name: Annotated[
        str,
        Parameter(
            title="Название платформы",
            description="Обновленное название платформы происхождения."
        )
    ]
    slug: Annotated[
        str,
        Parameter(
            title="Slug",
            description="Обновленный человекочитаемый идентификатор платформы."
        )
    ]


class OriginPlatformResponse(Struct):
    id: int
    name: str
    slug: str
    created_at: datetime
    updated_at: datetime


class OriginPlatformShortResponse(Struct):
    id: int
    name: str



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import Column, ForeignKey, Index
from sqlalchemy.dialects.postgresql import BIGINT, TEXT, ENUM, ARRAY, VARCHAR
from sqlalchemy.orm import relationship, Mapped

from .decorator.jsonb_dict import JSONBDict

from .enums import SyncTaskType, SyncTaskStatus

if TYPE_CHECKING:
    from .product_platform import ProductPlatform


class ProductPlatformSyncTask(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_sync_task", "product_platform_id", "task_type", "status"),
    )

    product_platform_id: Mapped[int] = Column(BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"), nullable=False)
    task_type: Mapped[SyncTaskType] = Column(ENUM(SyncTaskType, name="sync_task_type_enum"), nullable=False)
    task_meta: Mapped[dict] = Column(JSONBDict, nullable=True)
    routing_keys: Mapped[list[str]] = Column(ARRAY(VARCHAR(255)), default=[])
    sub_statuses: Mapped[dict] = Column(JSONBDict, default={})
    sub_infos: Mapped[dict] = Column(JSONBDict, default={})
    status: Mapped[SyncTaskStatus] = Column(ENUM(SyncTaskStatus, name="sync_task_status_enum"), nullable=False)
    info: Mapped[str] = Column(TEXT, nullable=True)

    product_platform: Mapped[ProductPlatform] = relationship(
        back_populates="sync_tasks",
        foreign_keys="ProductPlatformSyncTask.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="noload",
    )



import msgspec

from ..schemas import Locale, BaseResponse, VariantTypes, OptionTypes


class OptionName(msgspec.Struct, forbid_unknown_fields=True):
    locale: Locale
    value: str


class OptionComment(msgspec.Struct, forbid_unknown_fields=True):
    locale: Locale
    value: str


class Option(msgspec.Struct, forbid_unknown_fields=True):
    id: int
    type: OptionTypes
    order: int
    name: list[OptionName] | None
    comment: list[OptionComment] | None

    separate_content: int | None   # to bool, лишняя хня
    modifier_visible: int | None   # to bool, лишняя хня


class VariantName(msgspec.Struct, forbid_unknown_fields=True):
    locale: Locale
    value: str


class Variant(msgspec.Struct, forbid_unknown_fields=True):
    variant_id: int
    name: list[VariantName] | None
    type: VariantTypes
    rate: float
    is_default: bool
    visible: bool
    order: int


class VariantCreateRequest(msgspec.Struct, forbid_unknown_fields=True):
    name: list[VariantName]
    type: VariantTypes
    rate: int
    default: bool
    order: int


class VariantCreateResponse(msgspec.Struct, forbid_unknown_fields=True):
    variants: list[int]


class OptionDetails(msgspec.Struct, forbid_unknown_fields=True):
    id: int
    type: OptionTypes
    name: list[OptionName] | None
    comment: list[OptionComment] | None
    order: int
    no_default: bool
    separate_content: bool
    required: bool
    modifier_visible: bool
    variants: list[Variant] | None


class OptionCreateResponse(msgspec.Struct, forbid_unknown_fields=True):
    option_id: int

    parameter_id: int | None    # лишняя хня с тем же значением что и в option_id


class UpdateDeleteResponse(msgspec.Struct, forbid_unknown_fields=True):
    status: str


class ProductParametersResponse(BaseResponse, forbid_unknown_fields=True):
    content: list[Option] | None


class ProductParameterResponse(BaseResponse, forbid_unknown_fields=True):
    content: OptionDetails | None


class ProductParameterCreateRequest(msgspec.Struct, forbid_unknown_fields=True):
    product_id: int
    name: list[OptionName] | None
    comment: list[OptionComment] | None
    type: OptionTypes
    separate_content: bool
    required: bool
    modifier_visible: bool
    order: int
    variants: list[VariantCreateRequest] | None


class ProductParameterCreateResponse(BaseResponse, forbid_unknown_fields=True):
    content: OptionCreateResponse | None


class ProductParameterUpdateRequest(msgspec.Struct, forbid_unknown_fields=True):
    option_id: int
    name: list[OptionName] | None
    comment: list[OptionComment] | None
    type: OptionTypes
    separate_content: bool
    required: bool
    modifier_visible: bool
    order: int


class ProductParameterUpdateResponse(BaseResponse, forbid_unknown_fields=True):
    content: UpdateDeleteResponse | None


class ProductParameterDeleteResponse(BaseResponse, forbid_unknown_fields=True):
    content: UpdateDeleteResponse | None


class ProductParameterVariantCreateRequest(msgspec.Struct, forbid_unknown_fields=True):
    variants: list[VariantCreateRequest]


class ProductParameterVariantCreateResponse(BaseResponse, forbid_unknown_fields=True):
    content: VariantCreateResponse | None


class ProductParameterVariantUpdateRequest(msgspec.Struct, forbid_unknown_fields=True):
    name: list[VariantName] | None
    type: VariantTypes
    rate: int
    default: bool
    visible: bool
    order: int


class ProductParameterVariantUpdateResponse(BaseResponse, forbid_unknown_fields=True):
    content: UpdateDeleteResponse | None


class ProductParameterVariantDeleteResponse(BaseResponse, forbid_unknown_fields=True):
    content: UpdateDeleteResponse | None



from .filters_and_sorting_schema import (SortType, FieldFilter, SortingRulesSchema, FilteringRulesSchema)

__all__ = (
    'SortType',
    'FieldFilter',
    'SortingRulesSchema',
    'FilteringRulesSchema'
)



from datetime import datetime
from typing import Annotated, Dict, Any

from litestar.params import Parameter
from msgspec import Struct


class PlatformCreateRequest(Struct, forbid_unknown_fields=True):
    name: Annotated[
        str,
        Parameter(
            title="Name",
            description="The name of the platform."
        )
    ]
    description: Annotated[
        str,
        Parameter(
            title="Description",
            description="The description of the platform."
        )
    ]
    custom_fields_template: Annotated[
        Dict[str, Any],
        Parameter(
            title="Custom Fields Template",
            description="The custom fields template of the platform."
        )
    ]
    slug: Annotated[
        str,
        Parameter(
            title="Slug",
            description="The slug of the platform."
        )
    ]
    is_external: Annotated[
        bool,
        Parameter(
            title="Is External",
            description="Whether the platform is external."
        )
    ]


class PlatformUpdateRequest(Struct, forbid_unknown_fields=True):
    name: str
    description: str
    custom_fields_template: Dict[str, Any]
    slug: str
    is_external: bool


class PlatformResponse(Struct):
    id: int
    name: str
    description: str
    custom_fields_template: Dict[str, Any]
    slug: str
    is_external: bool
    created_at: datetime
    updated_at: datetime



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct


class ProductPlatformDiscountCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[
        int,
        Parameter(
            title="Product Platform ID",
            description="The ID of the product platform."
        )
    ]
    unit_for_discount: Annotated[
        int,
        Parameter(
            title="Unit For Discount",
            description="The unit for discount."
        )
    ]
    discount: Annotated[
        int,
        Parameter(
            title="Discount",
            description="The discount."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="The is enabled."
        )
    ]


class ProductPlatformDiscountUpdateRequest(Struct, forbid_unknown_fields=True):
    unit_for_discount: Annotated[
        int,
        Parameter(
            title="Unit For Discount",
            description="The unit for discount."
        )
    ]
    discount: Annotated[
        int,
        Parameter(
            title="Discount",
            description="The discount."
        )
    ]
    is_enabled: Annotated[
        bool,
        Parameter(
            title="Is Enabled",
            description="The is enabled."
        )
    ]


class ProductPlatformDiscountResponse(Struct):
    id: int
    product_platform_id: int
    version: int
    unit_for_discount: int
    discount: int
    is_enabled: bool
    created_at: datetime
    updated_at: datetime



from typing import Optional, List
from stofory_sdk.catalog.models.enums import ProductType, ProductKind, Currency, ParameterType, ModifierOperator, \
    PricingType
import msgspec

__all__ = ('CatalogProductsRequest', 'CatalogProductsRegion', 'CatalogProductsResponse', 'ProductInfoResponse')

from stofory_sdk.lib.schemas.filters_and_sorting_schema import SortingRulesSchema, SortType


class CatalogProductsRequest(SortingRulesSchema, forbid_unknown_fields=True):
    """
    Модель запроса для получения списка товаров из каталога с фильтрацией и пагинацией.
    """
    platform_id: int  # Идентификатор платформы для отображения товаров
    lang_id: int  # Идентификатор языка для отображения информации о товарах
    currency: Currency  # Идентификатор валюты для отображения информации о товарах

    origin_platform_product: Optional[int] = None  # Идентификатор платформы товара для фильтрации
    type_product: Optional[ProductType] = None  # Тип товара для фильтрации (например, 'physical', 'digital')
    kind_product: Optional[ProductKind] = None  # Категория товара для фильтрации (например, 'key', 'account')
    region_sell: Optional[int] = None  # Регион продажи товара для фильтрации (например, 'RU', 'US')

    price_min: Optional[float] = None  # Минимальная цена товара для фильтрации (включительно)
    price_max: Optional[float] = None  # Максимальная цена товара для фильтрации (включительно)

    is_discount: Optional[bool] = None  # Фильтр по наличию скидки (True - со скидкой, False - без скидки)
    is_enabled: Optional[bool] = None  # Фильтр по наличию на складе (True - в наличии, False - нет в наличии)

    page: int = 1  # Номер страницы для пагинации (начиная с 1)
    page_size: int = 10  # Количество товаров на странице (минимум 10, максимум 100)

    search_string: Optional[str] = None  # Значение для поиска

    s__name: Optional[SortType] = None
    s__price: Optional[SortType] = None
    s__platform: Optional[SortType] = None
    s__product_type: Optional[SortType] = None
    s__product_kind: Optional[SortType] = None
    s__region: Optional[SortType] = None


class CatalogProductsRegion(msgspec.Struct):
    id: int
    alpha2: str


class InfoPricing(msgspec.Struct):
    id: int
    pricing_type: PricingType
    price: int | None
    price_per_unit: int | None
    min_quantity: int | None
    max_quantity: int | None
    unit_name: str | None
    currency: Currency


class CatalogProductsResponse(msgspec.Struct, kw_only=True):
    id: int
    lang_id: int
    name: str
    image_domain: str
    image_relative_path: str
    pricing: InfoPricing
    currency: Currency
    is_enabled_discount: bool
    discount: float
    platform: str
    product_type: ProductType
    product_kind: ProductKind
    region_info: List[CatalogProductsRegion]


class InfoImage(msgspec.Struct):
    id: int
    domain: str
    relative_path: str


class InfoParameterOption(msgspec.Struct):
    id: int
    name: str
    order: int
    is_default: bool
    is_enabled: bool
    modifier_operator: ModifierOperator
    modifier_value: int


class InfoParameter(msgspec.Struct):
    id: int
    name: str
    comment: str
    order: int
    is_required: bool
    parameter_type: ParameterType
    options: List[InfoParameterOption]


class ProductInfoResponse(msgspec.Struct, kw_only=True):
    id: int
    name: str
    platform: str
    description: str
    discount: int
    pricing: InfoPricing
    images: List[InfoImage]
    region_info: List[CatalogProductsRegion]
    parameters: List[InfoParameter]



from .preset_schema import PlatformInfoResponse, LanguagesInfoResponse, CatalogFilterResponse
from .products_schema import CatalogProductsRequest, CatalogProductsRegion, CatalogProductsResponse, ProductInfoResponse

__all__ = (
    PlatformInfoResponse,
    LanguagesInfoResponse,
    CatalogFilterResponse,

    CatalogProductsRequest,
    CatalogProductsRegion,
    CatalogProductsResponse,
    ProductInfoResponse

)



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Index
from sqlalchemy.dialects.postgresql import BIGINT, VARCHAR, ENUM
from sqlalchemy.orm import Mapped, relationship, mapped_column

from .enums import PricingType, Currency

if TYPE_CHECKING:
    from .product_platform import ProductPlatform


class ProductPlatformPricing(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_pricing_product_platform_id", "product_platform_id"),
    )

    product_platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"),
                                                          nullable=False, unique=True)
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    pricing_type: Mapped[PricingType] = mapped_column(ENUM(PricingType, name="pricing_type_enum"), nullable=False)
    price: Mapped[int] = mapped_column(BIGINT, nullable=True)
    price_per_unit: Mapped[int] = mapped_column(BIGINT, nullable=True)
    min_quantity: Mapped[int] = mapped_column(BIGINT, nullable=True)
    max_quantity: Mapped[int] = mapped_column(BIGINT, nullable=True)
    unit_name: Mapped[str] = mapped_column(VARCHAR(255), nullable=True)
    currency: Mapped[Currency] = mapped_column(ENUM(Currency, name="currency_enum"), nullable=False)

    product_platform: Mapped[ProductPlatform] = relationship(
        back_populates="price",
        foreign_keys="ProductPlatformPricing.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="joined"
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Enum, Index
from sqlalchemy.dialects.postgresql import BIGINT, VARCHAR
from sqlalchemy.orm import Mapped, relationship, mapped_column

if TYPE_CHECKING:
    from .category import Category
    from .localization import Localization


class CategoryName(BigIntAuditBase):
    __table_args__ = (
        Index("idx_category_name_category_id", "category_id"),
        Index("idx_category_name_localization_id", "localization_id"),
    )

    category_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("category.id", ondelete="CASCADE"), nullable=False)
    localization_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("localization.id", ondelete="CASCADE"), nullable=False)
    value: Mapped[str] = mapped_column(VARCHAR(255), nullable=False)

    category: Mapped[Category] = relationship(
        back_populates="names",
        foreign_keys="CategoryName.category_id",
        innerjoin=True,
        uselist=False,
        lazy="noload"
    )

    locale: Mapped[Localization] = relationship(
        back_populates="category_names",
        foreign_keys="CategoryName.localization_id",
        innerjoin=True,
        uselist=False,
        lazy="joined"
    )



from datetime import datetime
from typing import Annotated, Any

from litestar.params import Parameter
from msgspec import Struct

from . import (
    # ProductPlatformCreateRequest,     # base struct for create request schemas?

    ProductPlatformNameCreateRequest,
    ProductPlatformDescriptionCreateRequest,
    ProductPlatformCategoryCreateRequest,
    ProductPlatformPricingCreateRequest,
    ProductPlatformBonusCreateRequest,
    ProductPlatformDiscountCreateRequest,
    ProductPlatformGuaranteeCreateRequest,
    ProductPlatformNameUpdateRequest,
    ProductPlatformDescriptionUpdateRequest,
    ProductPlatformCategoryUpdateRequest,
    ProductPlatformPricingUpdateRequest,
    ProductPlatformBonusUpdateRequest,
    ProductPlatformDiscountUpdateRequest,
    ProductPlatformGuaranteeUpdateRequest,
    ProductPlatformImageCreateRequest,
    ProductPlatformImageUpdateRequest,
    ProductPlatformVideoCreateRequest,
    ProductPlatformVideoUpdateRequest,
    ProductPlatformParameterCreateRequest,
    ParameterNameCreateRequest,
    ParameterCommentCreateRequest,
    ParameterOptionCreateRequest,
    ParameterOptionNameCreateRequest,
    ProductPlatformParameterUpdateRequest,
    ParameterNameUpdateRequest,
    ParameterCommentUpdateRequest,
    ParameterOptionUpdateRequest,
    ParameterOptionNameUpdateRequest, CategoryCreateRequest, CategoryUpdateRequest,
)
from .category_name_schema import CategoryNameCreateRequest, CategoryNameUpdateRequest


# Wrapper for all isolated update request schemas
class GenericUpdateRequest[T, U](Struct, forbid_unknown_fields=True):
    id: int | None = None
    update: T | None = None
    create: U | None = None

    def __post_init__(self):
        if self.update is not None and self.create is not None:
            raise ValueError("Only one of 'update' or 'create' should be specified")

        if self.update is None and self.create is None:
            raise ValueError("One of 'update' or 'create' should be specified")

        if self.update is not None and self.id is None:
            raise ValueError("Id should be specified for 'update'")

        if self.create is not None and self.id is not None:
            raise ValueError("Id should not be specified for 'create'")


# Basic group
class BasicGroupCreateRequest(Struct, forbid_unknown_fields=True):
    names: list[ProductPlatformNameCreateRequest]
    descriptions: list[ProductPlatformDescriptionCreateRequest]
    categories: list[ProductPlatformCategoryCreateRequest]
    price: ProductPlatformPricingCreateRequest
    bonus: ProductPlatformBonusCreateRequest | None
    discount: ProductPlatformDiscountCreateRequest | None
    guarantee: ProductPlatformGuaranteeCreateRequest | None


class BasicGroupUpdateRequest(Struct, forbid_unknown_fields=True):
    is_visible: bool
    names: list[GenericUpdateRequest[ProductPlatformNameUpdateRequest, ProductPlatformNameCreateRequest]]
    descriptions: list[GenericUpdateRequest[ProductPlatformDescriptionUpdateRequest, ProductPlatformDescriptionCreateRequest]]
    categories: list[GenericUpdateRequest[ProductPlatformCategoryUpdateRequest, ProductPlatformCategoryCreateRequest]]
    price: GenericUpdateRequest[ProductPlatformPricingUpdateRequest, ProductPlatformPricingCreateRequest]
    bonus: GenericUpdateRequest[ProductPlatformBonusUpdateRequest, ProductPlatformBonusCreateRequest] | None
    discount: GenericUpdateRequest[ProductPlatformDiscountUpdateRequest, ProductPlatformDiscountCreateRequest] | None
    guarantee: GenericUpdateRequest[ProductPlatformGuaranteeUpdateRequest, ProductPlatformGuaranteeCreateRequest] | None


# Gallery group
class GalleryGroupCreateRequest(Struct, forbid_unknown_fields=True):
    images: list[ProductPlatformImageCreateRequest]
    videos: list[ProductPlatformVideoCreateRequest]


class GalleryGroupUpdateRequest(Struct, forbid_unknown_fields=True):
    images: list[GenericUpdateRequest[ProductPlatformImageUpdateRequest, ProductPlatformImageCreateRequest]]
    videos: list[GenericUpdateRequest[ProductPlatformVideoUpdateRequest, ProductPlatformVideoCreateRequest]]


# Parameters group
class GroupParameterOptionCreateRequest(ParameterOptionCreateRequest, forbid_unknown_fields=True):
    names: list[GenericUpdateRequest[None, ParameterOptionNameCreateRequest]]


class GroupParameterCreateRequest(ProductPlatformParameterCreateRequest, forbid_unknown_fields=True):
    names: list[GenericUpdateRequest[None, ParameterNameCreateRequest]]
    comments: list[GenericUpdateRequest[None, ParameterCommentCreateRequest]]
    options: list[GenericUpdateRequest[None, GroupParameterOptionCreateRequest]]


class ParametersGroupCreateRequest(Struct, forbid_unknown_fields=True):
    parameters: list[GenericUpdateRequest[None, GroupParameterCreateRequest]]


class GroupParameterOptionUpdateRequest(ParameterOptionUpdateRequest, forbid_unknown_fields=True):
    names: list[GenericUpdateRequest[ParameterOptionNameUpdateRequest, ParameterOptionNameCreateRequest]]


class GroupParameterUpdateRequest(ProductPlatformParameterUpdateRequest, forbid_unknown_fields=True):
    names: list[GenericUpdateRequest[ParameterNameUpdateRequest, ParameterNameCreateRequest]]
    comments: list[GenericUpdateRequest[ParameterCommentUpdateRequest, ParameterCommentCreateRequest]]
    options: list[GenericUpdateRequest[GroupParameterOptionUpdateRequest, GroupParameterOptionCreateRequest]]


class ParametersGroupUpdateRequest(Struct, forbid_unknown_fields=True):
    parameters: list[GenericUpdateRequest[GroupParameterUpdateRequest, GroupParameterCreateRequest]]


class CategoryGroupCreateRequest(CategoryCreateRequest, forbid_unknown_fields=True, kw_only=True):
    names: list[CategoryNameCreateRequest]


class CategoryGroupUpdateRequest(CategoryUpdateRequest, forbid_unknown_fields=True, kw_only=True):
    names: list[GenericUpdateRequest[CategoryNameUpdateRequest, CategoryNameCreateRequest]]



from datetime import datetime
from typing import Annotated, Optional

from litestar.params import Parameter
from msgspec import Struct

from stofory_sdk.catalog.models.enums import LocationType, PartOfWorldType


class RegionCreateRequest(Struct, forbid_unknown_fields=True):
    name_ru: Annotated[
        str,
        Parameter(
            title="Название на русском",
            description="Название региона на русском языке.",
        )
    ]
    name_en: Annotated[
        str,
        Parameter(
            title="Название на английском",
            description="Название региона на английском языке.",
        )
    ]
    alpha2: Annotated[
        str,
        Parameter(
            title="Alpha2",
            description="Код региона в формате Alpha2 (2 символа).",
            pattern="^[A-Z]{2}$"
        )
    ]
    alpha3: Annotated[
        str,
        Parameter(
            title="Alpha3",
            description="Код региона в формате Alpha3 (3 символа).",
            pattern="^[A-Z]{3}$"
        )
    ]
    iso: Annotated[
        int,
        Parameter(
            title="ISO",
            description="ISO-код региона (числовой код).",
        )
    ]
    part_of_world: Annotated[
        PartOfWorldType | None,
        Parameter(
            title="Часть света",
            description="Часть света, к которой относится регион.",
        )
    ]
    location: Annotated[
        LocationType | None,
        Parameter(
            title="Расположение",
            description="Географическое расположение региона.",
        )
    ]


class RegionUpdateRequest(Struct, forbid_unknown_fields=True):
    name_ru: Annotated[
        Optional[str],
        Parameter(
            title="Название на русском",
            description="Название региона на русском языке.",
        )
    ]
    name_en: Annotated[
        Optional[str],
        Parameter(
            title="Название на английском",
            description="Название региона на английском языке.",
        )
    ]
    iso: Annotated[
        Optional[int],
        Parameter(
            title="ISO",
            description="ISO-код региона (числовой код).",
        )
    ]
    part_of_world: Annotated[
        Optional[PartOfWorldType],
        Parameter(
            title="Часть света",
            description="Часть света, к которой относится регион.",
        )
    ]
    location: Annotated[
        Optional[LocationType],
        Parameter(
            title="Расположение",
            description="Географическое расположение региона.",
        )
    ]


class RegionResponse(Struct):
    id: int
    name_ru: str
    name_en: str
    alpha2: str
    alpha3: str
    iso: int
    part_of_world: PartOfWorldType | None
    location: LocationType | None
    created_at: datetime
    updated_at: datetime


class RegionShortResponse(Struct):
    id: int
    name_ru: str
    name_en: str
    alpha2: str
    alpha3: str



from datetime import datetime
from enum import StrEnum
from typing import Annotated, Any, Optional, List

from msgspec import Meta

from ..lib import BaseStruct
from ..schemas import CurrencyDigiseller

__all__ = ('CaptchaResponse', 'PartnerRegistrationRequest', 'PartnerRegistrationResponse', 'PermissionsResponse',
           'PaymentVerificationResponse', 'SaleInfoResponse', 'UniqueCodeState', 'BuyerInfo', 'Content')


class CaptchaResponse(BaseStruct, forbid_unknown_fields=True):
    retval: int  # код выполнения запроса
    retdesc: Optional[str]  # описание ошибки (если retval != 0)
    id_seller: Optional[int] = None  # идентификатор продавца
    id_request: Optional[int] = None  # номер запроса для следующего шага
    img_url: Optional[str] = None  # URL изображения капчи


class PartnerRegistrationRequest(BaseStruct, forbid_unknown_fields=True):
    id_seller: Annotated[int, Meta(gt=0)]  # Идентификатор продавца
    id_request: int  # Номер запроса, полученный в предыдущем шаге
    turing_num: int  # Введенное партнером проверочное число (капча)
    r_email: str  # Email пользователя (партнера)
    r_redirect_url: str  # URL для перенаправления


class PartnerRegistrationResponse(BaseStruct, forbid_unknown_fields=True):
    retval: int  # Код выполнения запроса
    retdesc: str  # Текст ошибки при retval != 0
    id_agent: Optional[int] = None  # ID агента (0, если регистрация еще не завершена)


class PermissionsResponse(BaseStruct, forbid_unknown_fields=True):
    description_create: bool = False
    description_edit: bool = False
    description_view: bool = False
    content_upload: bool = False
    content_change: bool = False
    content_delete: bool = False
    gallery_add: bool = False
    gallery_change: bool = False
    option_read: bool = False
    option_add: bool = False
    option_edit: bool = False
    option_delete: bool = False
    view_operations: bool = False
    view_invoice: bool = False
    sales_statistics_view: bool = False
    agent_sales_statistics_view: bool = False
    view_personal_offer: bool = False
    read_commission_template: bool = False
    create_commission_template: bool = False
    edit_commission_template: bool = False
    delete_commission_template: bool = False
    apply_commission_template: bool = False
    debates_read: bool = False
    debates_write: bool = False
    debates_delete: bool = False
    token_get_perms: bool = False


class UniqueCodeState(BaseStruct, forbid_unknown_fields=True):
    state: int
    options: Any = None  # Кто ты воин?
    date_check: Optional[datetime] = None  # строка. необязательное поле
    date_delivery: Optional[datetime] = None  # строка. необязательное поле
    date_refuted: Optional[datetime] = None  # строка. необязательное поле
    date_confirmed: Optional[datetime] = None  # строка. необязательное поле


class PaymentVerificationOption(BaseStruct, forbid_unknown_fields=True):
    id: int  # атрибут id - уникальный идентификатор параметра (кого хуя они option пишут)
    name: str
    value: str
    variant_id: Optional[int] = None  # целое число. может быть пустым (только для полей 'radio' и 'select')


class PaymentVerificationResponse(BaseStruct, kw_only=True, forbid_unknown_fields=True):
    retval: int
    retdesc: Optional[str]
    inv: int
    id_goods: int
    amount: float
    type_curr: CurrencyDigiseller
    amount_usd: float

    profit: Optional[float] = None  # Так как в одном из примеров profit пустое значение
    date_pay: Optional[datetime] = None
    email: Optional[str]
    method: Optional[str] = None  # Ебать его не было в документации - КЛЯНУСЬ НЕ БЫЛО. Видимо метод оплаты
    name_invoice: Optional[str]
    lang: Optional[str]

    agent_id: int
    agent_percent: float
    query_string: Optional[str]  # base64-закодированная строка

    unit_goods: Optional[int] = None  # только для товаров с нефиксированной ценой
    cnt_goods: Optional[float] = None  # только для товаров с нефиксированной ценой
    promo_code: Optional[str] = None  # только если при оплате был использован промо-код
    bonus_code: Optional[str] = None  # только если после оплате был выдан промо-код
    cart_uid: Optional[str] = None  # только если товар был добавлен и оплачен через корзину

    unique_code_state: Optional[
        UniqueCodeState] = None  # необязательное поле уникальный код (состояние и даты проверки)
    options: Optional[List[PaymentVerificationOption]] = None  # только для товаров с доп. параметрами


# Модель для поля feedback
class UserFeedback(StrEnum):
    POSITIVE = 'positive'
    NEGATIVE = 'negative'


class Feedback(BaseStruct, forbid_unknown_fields=True):
    deleted: bool
    feedback: str
    feedback_type: UserFeedback
    comment: str


# Модель для информации о покупателе
class BuyerInfo(BaseStruct, kw_only=True, forbid_unknown_fields=True):
    payment_method: str
    payment_aggregator: str
    account: Optional[str] = None
    email: str
    phone: Optional[str] = None
    skype: Optional[str] = None
    whatsapp: Optional[str] = None
    ip_address: Optional[str]


class ContentOption(BaseStruct, forbid_unknown_fields=True):
    id: int  # атрибут id - уникальный идентификатор параметра
    name: str
    user_data: str
    user_data_id: Optional[int] = None  # целое число. может быть пустым (только для полей 'radio' и 'select')


class PaymentHoldState(StrEnum):
    FREE = 'free'
    LOCKED = 'locked'
    EMPTY = ''


# Основная модель content
class Content(BaseStruct, forbid_unknown_fields=True, kw_only=True):
    item_id: int
    cart_uid: Optional[str] = None  # только если товар был добавлен и оплачен через корзину
    name: Optional[str]
    amount: float  # сумма, зачисленная на ваш счет обязательно проверяйте данное значение!
    currency_type: CurrencyDigiseller
    invoice_state: int

    purchase_date: datetime
    date_pay: Optional[datetime] = None  # строка. необязательное поле

    agent_id: Optional[int] = None  # ID партнера, если продажа была совершена с участием партнера
    agent_percent: Optional[float] = None  # процент партнерского вознаграждения
    query_string: Optional[str]  # # base64-закодированная строка,

    unit_goods: Optional[int] = None  # только для товаров с нефиксированной ценой
    cnt_goods: Optional[float] = None  # только для товаров с нефиксированной ценой
    promo_code: Optional[str] = None  # только если при оплате был использован промо-код
    bonus_code: Optional[str] = None  # только если после оплате был выдан промо-код

    feedback: Optional[Feedback] = None  # отзыв покупателя
    unique_code_state: Optional[
        UniqueCodeState] = None  # необязательное поле уникальный код (состояние и даты проверки)
    options: Optional[List[ContentOption]] = None
    buyer_info: BuyerInfo
    referer: Optional[str] = None  # Ебать его не было в документации - КЛЯНУСЬ НЕ БЫЛО.
    owner: int
    day_lock: int
    lock_state: Optional[PaymentHoldState]
    profit: Optional[float] = None  # Зачислено на счёт
    agent_fee: Optional[float] = None  # Коммисия агента


class Error(BaseStruct, forbid_unknown_fields=True):
    code: str
    message: str


# Модель для ответа SaleInfoResponse
class SaleInfoResponse(BaseStruct, forbid_unknown_fields=True):
    retval: int
    retdesc: Optional[str]
    errors: Optional[List[Error]]
    content: Optional[Content]



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import Column, ForeignKey, Index
from sqlalchemy.dialects.postgresql import BIGINT, VARCHAR
from sqlalchemy.orm import Mapped, relationship, mapped_column

from .decorator.jsonb_dict import JSONBDict

if TYPE_CHECKING:
    from .product import Product
    from .platform import Platform
    from .product_platform import ProductPlatform


class ProductPlatformExternalConnection(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_external_connection", "product_platform_id", "platform_id", "product_id"),
    )

    product_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product.id"), nullable=False)
    platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("platform.id"), nullable=False)
    product_platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"), nullable=False, unique=True)
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    external_id: Mapped[str] = Column(VARCHAR(255), nullable=False)
    external_meta: Mapped[dict] = Column(JSONBDict, default={})

    product: Mapped[Product] = relationship(
        back_populates="external_connections",
        foreign_keys="ProductPlatformExternalConnection.product_id",
        innerjoin=True,
        uselist=False,
        lazy="noload",
    )

    platform: Mapped[Platform] = relationship(
        back_populates="external_connections",
        foreign_keys="ProductPlatformExternalConnection.platform_id",
        innerjoin=True,
        uselist=False,
        lazy="noload",
    )

    product_platform: Mapped[ProductPlatform] = relationship(
        back_populates="external_connections",
        foreign_keys="ProductPlatformExternalConnection.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="noload",
    )



from .schemas import (
    LimitationTypes,
    ProductUnfixedLimitation,
    ProductUnfixedPrice,
    ProductPreorder,
    AddVideoResult,
    CreateUniquefixedRequest,
    CreateUniqueunfixedRequest,
    AddVideosRequest,
    AddVideosResponse,
    UpdateStatusMassRequest,
    UpdateStatusMassResponse,
    UpdatePriceMassResponse,
    AsyncTaskStatusResponse,
    CreateUniquefixedResponse,
    CreateUniqueunfixedResponse,
    UpdateUniquefixedResponse,
    UpdateUniqueunfixedResponse,
    UploadImagesResponse,
    GalleryTypes,
    EditGalleryRequest,
    UpdatePriceMassRequest,
    AddProductToCategoryResponse,
    Platforms,
    PlatformCategoriesResponse,
    PlatformSubcategoriesResponse,
)


__all__ = (
    "LimitationTypes",
    "ProductUnfixedLimitation",
    "ProductUnfixedPrice",
    "ProductPreorder",
    "AddVideoResult",
    "CreateUniquefixedRequest",
    "CreateUniqueunfixedRequest",
    "AddVideosRequest",
    "AddVideosResponse",
    "UpdateStatusMassRequest",
    "UpdateStatusMassResponse",
    "UpdatePriceMassResponse",
    "AsyncTaskStatusResponse",
    "CreateUniquefixedResponse",
    "CreateUniqueunfixedResponse",
    "UpdateUniquefixedResponse",
    "UpdateUniqueunfixedResponse",
    "UploadImagesResponse",
    "GalleryTypes",
    "EditGalleryRequest",
    "UpdatePriceMassRequest",
    "AddProductToCategoryResponse",
    "Platforms",
    "PlatformCategoriesResponse",
    "PlatformSubcategoriesResponse",
)



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import Enum
from sqlalchemy.dialects.postgresql import BOOLEAN, ENUM
from sqlalchemy.orm import Mapped, mapped_column, relationship

from .enums import Locale

if TYPE_CHECKING:
    from .product_platform_name import ProductPlatformName
    from .category_name import CategoryName
    from .product_platform_description import ProductPlatformDescription
    from .parameter_name import ParameterName
    from .parameter_comment import ParameterComment
    from .parameter_option_name import ParameterOptionName


class Localization(BigIntAuditBase):
    locale: Mapped[Locale] = mapped_column(ENUM(Locale, name="locale_enum"), nullable=False, unique=True)
    is_enabled: Mapped[bool] = mapped_column(BOOLEAN, default=True)

    product_platform_names: Mapped[list[ProductPlatformName]] = relationship(
        back_populates="locale",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    category_names: Mapped[list[CategoryName]] = relationship(
        back_populates="locale",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    product_platform_descriptions: Mapped[list[ProductPlatformDescription]] = relationship(
        back_populates="locale",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    parameter_names: Mapped[list[ParameterName]] = relationship(
        back_populates="locale",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    parameter_comments: Mapped[list[ParameterComment]] = relationship(
        back_populates="locale",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    parameter_option_names: Mapped[list[ParameterOptionName]] = relationship(
        back_populates="locale",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )
